BEGIN_PROVIDER [integer, degree_max_integration_lebedev]
 BEGIN_DOC
! integrate correctly a polynom of order "degree_max_integration_lebedev"
 ! needed for the angular integration according to LEBEDEV formulae
 END_DOC
 implicit none
 degree_max_integration_lebedev= 15

END_PROVIDER

BEGIN_PROVIDER [integer, n_points_integration_angular_lebedev]
 BEGIN_DOC
! Number of points needed for the angular integral
 END_DOC
 implicit none
 if (degree_max_integration_lebedev == 3)then
  n_points_integration_angular_lebedev = 6
 else if (degree_max_integration_lebedev == 5)then
  n_points_integration_angular_lebedev = 14
 else if (degree_max_integration_lebedev == 7)then
  n_points_integration_angular_lebedev = 26
 else if (degree_max_integration_lebedev == 9)then
  n_points_integration_angular_lebedev = 38
 else if (degree_max_integration_lebedev == 11)then
  n_points_integration_angular_lebedev = 50
 else if (degree_max_integration_lebedev == 13)then
  n_points_integration_angular_lebedev = 74
 else if (degree_max_integration_lebedev == 15)then
  n_points_integration_angular_lebedev = 86
 else if (degree_max_integration_lebedev == 17)then
  n_points_integration_angular_lebedev = 110
 else if (degree_max_integration_lebedev == 19)then
  n_points_integration_angular_lebedev = 146
 else if (degree_max_integration_lebedev == 21)then
  n_points_integration_angular_lebedev = 170
 endif

END_PROVIDER

 BEGIN_PROVIDER [double precision, theta_angular_integration_lebedev, (n_points_integration_angular_lebedev)]
&BEGIN_PROVIDER [double precision, phi_angular_integration_lebedev, (n_points_integration_angular_lebedev)]
&BEGIN_PROVIDER [double precision, weights_angular_integration_lebedev, (n_points_integration_angular_lebedev)]
 implicit none
 BEGIN_DOC
! Theta phi values together with the weights values for the angular integration :
! integral [dphi,dtheta] f(x,y,z) = 4 * pi * sum (1<i<n_points_integration_angular_lebedev) f(xi,yi,zi)
! Note that theta and phi are in DEGREES !!
 END_DOC
 if (degree_max_integration_lebedev == 3)then
 theta_angular_integration_lebedev(1) = 0.000000000000000d0
 theta_angular_integration_lebedev(2) = 180.000000000000000d0
 theta_angular_integration_lebedev(3) = 90.000000000000000d0
 theta_angular_integration_lebedev(4) = -90.000000000000000d0
 theta_angular_integration_lebedev(5) = 90.000000000000000d0
 theta_angular_integration_lebedev(6) = 90.000000000000000d0

 phi_angular_integration_lebedev(1) = 90.000000000000000d0
 phi_angular_integration_lebedev(2) = 90.000000000000000d0
 phi_angular_integration_lebedev(3) = 90.000000000000000d0
 phi_angular_integration_lebedev(4) = 90.000000000000000d0
 phi_angular_integration_lebedev(5) = 0.000000000000000d0
 phi_angular_integration_lebedev(6) = 180.000000000000000d0

 weights_angular_integration_lebedev(1) = 0.166666666666667d0
 weights_angular_integration_lebedev(2) = 0.166666666666667d0
 weights_angular_integration_lebedev(3) = 0.166666666666667d0
 weights_angular_integration_lebedev(4) = 0.166666666666667d0
 weights_angular_integration_lebedev(5) = 0.166666666666667d0
 weights_angular_integration_lebedev(6) = 0.166666666666667d0
 else if( degree_max_integration_lebedev == 5)then
 theta_angular_integration_lebedev(1) = 0.000000000000000d0
 theta_angular_integration_lebedev(2) = 180.000000000000000d0
 theta_angular_integration_lebedev(3) = 90.000000000000000d0
 theta_angular_integration_lebedev(4) = -90.000000000000000d0
 theta_angular_integration_lebedev(5) = 90.000000000000000d0
 theta_angular_integration_lebedev(6) = 90.000000000000000d0
 theta_angular_integration_lebedev(7) = 45.000000000000000d0
 theta_angular_integration_lebedev(8) = 45.000000000000000d0
 theta_angular_integration_lebedev(9) = -45.000000000000000d0
 theta_angular_integration_lebedev(10) = -45.000000000000000d0
 theta_angular_integration_lebedev(11) = 135.000000000000000d0
 theta_angular_integration_lebedev(12) = 135.000000000000000d0
 theta_angular_integration_lebedev(13) = -135.000000000000000d0
 theta_angular_integration_lebedev(14) = -135.000000000000000d0

 phi_angular_integration_lebedev(1) = 90.000000000000000d0
 phi_angular_integration_lebedev(2) = 90.000000000000000d0
 phi_angular_integration_lebedev(3) = 90.000000000000000d0
 phi_angular_integration_lebedev(4) = 90.000000000000000d0
 phi_angular_integration_lebedev(5) = 0.000000000000000d0
 phi_angular_integration_lebedev(6) = 180.000000000000000d0
 phi_angular_integration_lebedev(7) = 54.735610317245346d0
 phi_angular_integration_lebedev(8) = 125.264389682754654d0
 phi_angular_integration_lebedev(9) = 54.735610317245346d0
 phi_angular_integration_lebedev(10) = 125.264389682754654d0
 phi_angular_integration_lebedev(11) = 54.735610317245346d0
 phi_angular_integration_lebedev(12) = 125.264389682754654d0
 phi_angular_integration_lebedev(13) = 54.735610317245346d0
 phi_angular_integration_lebedev(14) = 125.264389682754654d0

 weights_angular_integration_lebedev(1) = 0.066666666666667d0
 weights_angular_integration_lebedev(2) = 0.066666666666667d0
 weights_angular_integration_lebedev(3) = 0.066666666666667d0
 weights_angular_integration_lebedev(4) = 0.066666666666667d0
 weights_angular_integration_lebedev(5) = 0.066666666666667d0
 weights_angular_integration_lebedev(6) = 0.066666666666667d0
 weights_angular_integration_lebedev(7) = 0.075000000000000d0
 weights_angular_integration_lebedev(8) = 0.075000000000000d0
 weights_angular_integration_lebedev(9) = 0.075000000000000d0
 weights_angular_integration_lebedev(10) = 0.075000000000000d0
 weights_angular_integration_lebedev(11) = 0.075000000000000d0
 weights_angular_integration_lebedev(12) = 0.075000000000000d0
 weights_angular_integration_lebedev(13) = 0.075000000000000d0
 weights_angular_integration_lebedev(14) = 0.075000000000000d0

 else if (degree_max_integration_lebedev == 7)then


 theta_angular_integration_lebedev(1) = 0.000000000000000d0
 theta_angular_integration_lebedev(2) = 180.000000000000000d0
 theta_angular_integration_lebedev(3) = 90.000000000000000d0
 theta_angular_integration_lebedev(4) = -90.000000000000000d0
 theta_angular_integration_lebedev(5) = 90.000000000000000d0
 theta_angular_integration_lebedev(6) = 90.000000000000000d0
 theta_angular_integration_lebedev(7) = 90.000000000000000d0
 theta_angular_integration_lebedev(8) = 90.000000000000000d0
 theta_angular_integration_lebedev(9) = -90.000000000000000d0
 theta_angular_integration_lebedev(10) = -90.000000000000000d0
 theta_angular_integration_lebedev(11) = 0.000000000000000d0
 theta_angular_integration_lebedev(12) = 0.000000000000000d0
 theta_angular_integration_lebedev(13) = 180.000000000000000d0
 theta_angular_integration_lebedev(14) = 180.000000000000000d0
 theta_angular_integration_lebedev(15) = 45.000000000000000d0
 theta_angular_integration_lebedev(16) = -45.000000000000000d0
 theta_angular_integration_lebedev(17) = 135.000000000000000d0
 theta_angular_integration_lebedev(18) = -135.000000000000000d0
 theta_angular_integration_lebedev(19) = 45.000000000000000d0
 theta_angular_integration_lebedev(20) = 45.000000000000000d0
 theta_angular_integration_lebedev(21) = -45.000000000000000d0
 theta_angular_integration_lebedev(22) = -45.000000000000000d0
 theta_angular_integration_lebedev(23) = 135.000000000000000d0
 theta_angular_integration_lebedev(24) = 135.000000000000000d0
 theta_angular_integration_lebedev(25) = -135.000000000000000d0
 theta_angular_integration_lebedev(26) = -135.000000000000000d0

 phi_angular_integration_lebedev(1) = 90.000000000000000d0
 phi_angular_integration_lebedev(2) = 90.000000000000000d0
 phi_angular_integration_lebedev(3) = 90.000000000000000d0
 phi_angular_integration_lebedev(4) = 90.000000000000000d0
 phi_angular_integration_lebedev(5) = 0.000000000000000d0
 phi_angular_integration_lebedev(6) = 180.000000000000000d0
 phi_angular_integration_lebedev(7) = 45.000000000000000d0
 phi_angular_integration_lebedev(8) = 135.000000000000000d0
 phi_angular_integration_lebedev(9) = 45.000000000000000d0
 phi_angular_integration_lebedev(10) = 135.000000000000000d0
 phi_angular_integration_lebedev(11) = 45.000000000000000d0
 phi_angular_integration_lebedev(12) = 135.000000000000000d0
 phi_angular_integration_lebedev(13) = 45.000000000000000d0
 phi_angular_integration_lebedev(14) = 135.000000000000000d0
 phi_angular_integration_lebedev(15) = 90.000000000000000d0
 phi_angular_integration_lebedev(16) = 90.000000000000000d0
 phi_angular_integration_lebedev(17) = 90.000000000000000d0
 phi_angular_integration_lebedev(18) = 90.000000000000000d0
 phi_angular_integration_lebedev(19) = 54.735610317245346d0
 phi_angular_integration_lebedev(20) = 125.264389682754654d0
 phi_angular_integration_lebedev(21) = 54.735610317245346d0
 phi_angular_integration_lebedev(22) = 125.264389682754654d0
 phi_angular_integration_lebedev(23) = 54.735610317245346d0
 phi_angular_integration_lebedev(24) = 125.264389682754654d0
 phi_angular_integration_lebedev(25) = 54.735610317245346d0
 phi_angular_integration_lebedev(26) = 125.264389682754654d0

 weights_angular_integration_lebedev(1) = 0.047619047619048d0
 weights_angular_integration_lebedev(2) = 0.047619047619048d0
 weights_angular_integration_lebedev(3) = 0.047619047619048d0
 weights_angular_integration_lebedev(4) = 0.047619047619048d0
 weights_angular_integration_lebedev(5) = 0.047619047619048d0
 weights_angular_integration_lebedev(6) = 0.047619047619048d0
 weights_angular_integration_lebedev(7) = 0.038095238095238d0
 weights_angular_integration_lebedev(8) = 0.038095238095238d0
 weights_angular_integration_lebedev(9) = 0.038095238095238d0
 weights_angular_integration_lebedev(10) = 0.038095238095238d0
 weights_angular_integration_lebedev(11) = 0.038095238095238d0
 weights_angular_integration_lebedev(12) = 0.038095238095238d0
 weights_angular_integration_lebedev(13) = 0.038095238095238d0
 weights_angular_integration_lebedev(14) = 0.038095238095238d0
 weights_angular_integration_lebedev(15) = 0.038095238095238d0
 weights_angular_integration_lebedev(16) = 0.038095238095238d0
 weights_angular_integration_lebedev(17) = 0.038095238095238d0
 weights_angular_integration_lebedev(18) = 0.038095238095238d0
 weights_angular_integration_lebedev(19) = 0.032142857142857d0
 weights_angular_integration_lebedev(20) = 0.032142857142857d0
 weights_angular_integration_lebedev(21) = 0.032142857142857d0
 weights_angular_integration_lebedev(22) = 0.032142857142857d0
 weights_angular_integration_lebedev(23) = 0.032142857142857d0
 weights_angular_integration_lebedev(24) = 0.032142857142857d0
 weights_angular_integration_lebedev(25) = 0.032142857142857d0
 weights_angular_integration_lebedev(26) = 0.032142857142857d0

 else if (degree_max_integration_lebedev == 9)then


 theta_angular_integration_lebedev(1) = 0.000000000000000d0
 theta_angular_integration_lebedev(2) = 180.000000000000000d0
 theta_angular_integration_lebedev(3) = 90.000000000000000d0
 theta_angular_integration_lebedev(4) = -90.000000000000000d0
 theta_angular_integration_lebedev(5) = 90.000000000000000d0
 theta_angular_integration_lebedev(6) = 90.000000000000000d0
 theta_angular_integration_lebedev(7) = 45.000000000000000d0
 theta_angular_integration_lebedev(8) = 45.000000000000000d0
 theta_angular_integration_lebedev(9) = -45.000000000000000d0
 theta_angular_integration_lebedev(10) = -45.000000000000000d0
 theta_angular_integration_lebedev(11) = 135.000000000000000d0
 theta_angular_integration_lebedev(12) = 135.000000000000000d0
 theta_angular_integration_lebedev(13) = -135.000000000000000d0
 theta_angular_integration_lebedev(14) = -135.000000000000000d0
 theta_angular_integration_lebedev(15) = 62.632194841377327d0
 theta_angular_integration_lebedev(16) = -62.632194841377327d0
 theta_angular_integration_lebedev(17) = 117.367805158622687d0
 theta_angular_integration_lebedev(18) = -117.367805158622687d0
 theta_angular_integration_lebedev(19) = 27.367805158622673d0
 theta_angular_integration_lebedev(20) = -27.367805158622673d0
 theta_angular_integration_lebedev(21) = 152.632194841377355d0
 theta_angular_integration_lebedev(22) = -152.632194841377355d0
 theta_angular_integration_lebedev(23) = 0.000000000000000d0
 theta_angular_integration_lebedev(24) = 0.000000000000000d0
 theta_angular_integration_lebedev(25) = 180.000000000000000d0
 theta_angular_integration_lebedev(26) = 180.000000000000000d0
 theta_angular_integration_lebedev(27) = 0.000000000000000d0
 theta_angular_integration_lebedev(28) = 0.000000000000000d0
 theta_angular_integration_lebedev(29) = 180.000000000000000d0
 theta_angular_integration_lebedev(30) = 180.000000000000000d0
 theta_angular_integration_lebedev(31) = 90.000000000000000d0
 theta_angular_integration_lebedev(32) = 90.000000000000000d0
 theta_angular_integration_lebedev(33) = -90.000000000000000d0
 theta_angular_integration_lebedev(34) = -90.000000000000000d0
 theta_angular_integration_lebedev(35) = 90.000000000000000d0
 theta_angular_integration_lebedev(36) = 90.000000000000000d0
 theta_angular_integration_lebedev(37) = -90.000000000000000d0
 theta_angular_integration_lebedev(38) = -90.000000000000000d0

 phi_angular_integration_lebedev(1) = 90.000000000000000d0
 phi_angular_integration_lebedev(2) = 90.000000000000000d0
 phi_angular_integration_lebedev(3) = 90.000000000000000d0
 phi_angular_integration_lebedev(4) = 90.000000000000000d0
 phi_angular_integration_lebedev(5) = 0.000000000000000d0
 phi_angular_integration_lebedev(6) = 180.000000000000000d0
 phi_angular_integration_lebedev(7) = 54.735610317245346d0
 phi_angular_integration_lebedev(8) = 125.264389682754654d0
 phi_angular_integration_lebedev(9) = 54.735610317245346d0
 phi_angular_integration_lebedev(10) = 125.264389682754654d0
 phi_angular_integration_lebedev(11) = 54.735610317245346d0
 phi_angular_integration_lebedev(12) = 125.264389682754654d0
 phi_angular_integration_lebedev(13) = 54.735610317245346d0
 phi_angular_integration_lebedev(14) = 125.264389682754654d0
 phi_angular_integration_lebedev(15) = 90.000000000000000d0
 phi_angular_integration_lebedev(16) = 90.000000000000000d0
 phi_angular_integration_lebedev(17) = 90.000000000000000d0
 phi_angular_integration_lebedev(18) = 90.000000000000000d0
 phi_angular_integration_lebedev(19) = 90.000000000000000d0
 phi_angular_integration_lebedev(20) = 90.000000000000000d0
 phi_angular_integration_lebedev(21) = 90.000000000000000d0
 phi_angular_integration_lebedev(22) = 90.000000000000000d0
 phi_angular_integration_lebedev(23) = 27.367805158622673d0
 phi_angular_integration_lebedev(24) = 152.632194841377355d0
 phi_angular_integration_lebedev(25) = 27.367805158622673d0
 phi_angular_integration_lebedev(26) = 152.632194841377355d0
 phi_angular_integration_lebedev(27) = 62.632194841377327d0
 phi_angular_integration_lebedev(28) = 117.367805158622687d0
 phi_angular_integration_lebedev(29) = 62.632194841377327d0
 phi_angular_integration_lebedev(30) = 117.367805158622687d0
 phi_angular_integration_lebedev(31) = 27.367805158622673d0
 phi_angular_integration_lebedev(32) = 152.632194841377355d0
 phi_angular_integration_lebedev(33) = 27.367805158622673d0
 phi_angular_integration_lebedev(34) = 152.632194841377355d0
 phi_angular_integration_lebedev(35) = 62.632194841377327d0
 phi_angular_integration_lebedev(36) = 117.367805158622687d0
 phi_angular_integration_lebedev(37) = 62.632194841377327d0
 phi_angular_integration_lebedev(38) = 117.367805158622687d0

 weights_angular_integration_lebedev(1) = 0.009523809523810d0
 weights_angular_integration_lebedev(2) = 0.009523809523810d0
 weights_angular_integration_lebedev(3) = 0.009523809523810d0
 weights_angular_integration_lebedev(4) = 0.009523809523810d0
 weights_angular_integration_lebedev(5) = 0.009523809523810d0
 weights_angular_integration_lebedev(6) = 0.009523809523810d0
 weights_angular_integration_lebedev(7) = 0.032142857142857d0
 weights_angular_integration_lebedev(8) = 0.032142857142857d0
 weights_angular_integration_lebedev(9) = 0.032142857142857d0
 weights_angular_integration_lebedev(10) = 0.032142857142857d0
 weights_angular_integration_lebedev(11) = 0.032142857142857d0
 weights_angular_integration_lebedev(12) = 0.032142857142857d0
 weights_angular_integration_lebedev(13) = 0.032142857142857d0
 weights_angular_integration_lebedev(14) = 0.032142857142857d0
 weights_angular_integration_lebedev(15) = 0.028571428571429d0
 weights_angular_integration_lebedev(16) = 0.028571428571429d0
 weights_angular_integration_lebedev(17) = 0.028571428571429d0
 weights_angular_integration_lebedev(18) = 0.028571428571429d0
 weights_angular_integration_lebedev(19) = 0.028571428571429d0
 weights_angular_integration_lebedev(20) = 0.028571428571429d0
 weights_angular_integration_lebedev(21) = 0.028571428571429d0
 weights_angular_integration_lebedev(22) = 0.028571428571429d0
 weights_angular_integration_lebedev(23) = 0.028571428571429d0
 weights_angular_integration_lebedev(24) = 0.028571428571429d0
 weights_angular_integration_lebedev(25) = 0.028571428571429d0
 weights_angular_integration_lebedev(26) = 0.028571428571429d0
 weights_angular_integration_lebedev(27) = 0.028571428571429d0
 weights_angular_integration_lebedev(28) = 0.028571428571429d0
 weights_angular_integration_lebedev(29) = 0.028571428571429d0
 weights_angular_integration_lebedev(30) = 0.028571428571429d0
 weights_angular_integration_lebedev(31) = 0.028571428571429d0
 weights_angular_integration_lebedev(32) = 0.028571428571429d0
 weights_angular_integration_lebedev(33) = 0.028571428571429d0
 weights_angular_integration_lebedev(34) = 0.028571428571429d0
 weights_angular_integration_lebedev(35) = 0.028571428571429d0
 weights_angular_integration_lebedev(36) = 0.028571428571429d0
 weights_angular_integration_lebedev(37) = 0.028571428571429d0
 weights_angular_integration_lebedev(38) = 0.028571428571429d0

 else if (degree_max_integration_lebedev == 11)then


 theta_angular_integration_lebedev(1) = 0.000000000000000d0
 theta_angular_integration_lebedev(2) = 180.000000000000000d0
 theta_angular_integration_lebedev(3) = 90.000000000000000d0
 theta_angular_integration_lebedev(4) = -90.000000000000000d0
 theta_angular_integration_lebedev(5) = 90.000000000000000d0
 theta_angular_integration_lebedev(6) = 90.000000000000000d0
 theta_angular_integration_lebedev(7) = 90.000000000000000d0
 theta_angular_integration_lebedev(8) = 90.000000000000000d0
 theta_angular_integration_lebedev(9) = -90.000000000000000d0
 theta_angular_integration_lebedev(10) = -90.000000000000000d0
 theta_angular_integration_lebedev(11) = 0.000000000000000d0
 theta_angular_integration_lebedev(12) = 0.000000000000000d0
 theta_angular_integration_lebedev(13) = 180.000000000000000d0
 theta_angular_integration_lebedev(14) = 180.000000000000000d0
 theta_angular_integration_lebedev(15) = 45.000000000000000d0
 theta_angular_integration_lebedev(16) = -45.000000000000000d0
 theta_angular_integration_lebedev(17) = 135.000000000000000d0
 theta_angular_integration_lebedev(18) = -135.000000000000000d0
 theta_angular_integration_lebedev(19) = 45.000000000000000d0
 theta_angular_integration_lebedev(20) = 45.000000000000000d0
 theta_angular_integration_lebedev(21) = -45.000000000000000d0
 theta_angular_integration_lebedev(22) = -45.000000000000000d0
 theta_angular_integration_lebedev(23) = 135.000000000000000d0
 theta_angular_integration_lebedev(24) = 135.000000000000000d0
 theta_angular_integration_lebedev(25) = -135.000000000000000d0
 theta_angular_integration_lebedev(26) = -135.000000000000000d0
 theta_angular_integration_lebedev(27) = 45.000000000000000d0
 theta_angular_integration_lebedev(28) = 45.000000000000000d0
 theta_angular_integration_lebedev(29) = -45.000000000000000d0
 theta_angular_integration_lebedev(30) = -45.000000000000000d0
 theta_angular_integration_lebedev(31) = 135.000000000000000d0
 theta_angular_integration_lebedev(32) = 135.000000000000000d0
 theta_angular_integration_lebedev(33) = -135.000000000000000d0
 theta_angular_integration_lebedev(34) = -135.000000000000000d0
 theta_angular_integration_lebedev(35) = 71.565051177077990d0
 theta_angular_integration_lebedev(36) = -71.565051177077990d0
 theta_angular_integration_lebedev(37) = 71.565051177077990d0
 theta_angular_integration_lebedev(38) = -71.565051177077990d0
 theta_angular_integration_lebedev(39) = 108.434948822922010d0
 theta_angular_integration_lebedev(40) = -108.434948822922010d0
 theta_angular_integration_lebedev(41) = 108.434948822922010d0
 theta_angular_integration_lebedev(42) = -108.434948822922010d0
 theta_angular_integration_lebedev(43) = 18.434948822922017d0
 theta_angular_integration_lebedev(44) = 161.565051177078004d0
 theta_angular_integration_lebedev(45) = 18.434948822922017d0
 theta_angular_integration_lebedev(46) = 161.565051177078004d0
 theta_angular_integration_lebedev(47) = -18.434948822922017d0
 theta_angular_integration_lebedev(48) = -161.565051177078004d0
 theta_angular_integration_lebedev(49) = -18.434948822922017d0
 theta_angular_integration_lebedev(50) = -161.565051177078004d0

 phi_angular_integration_lebedev(1) = 90.000000000000000d0
 phi_angular_integration_lebedev(2) = 90.000000000000000d0
 phi_angular_integration_lebedev(3) = 90.000000000000000d0
 phi_angular_integration_lebedev(4) = 90.000000000000000d0
 phi_angular_integration_lebedev(5) = 0.000000000000000d0
 phi_angular_integration_lebedev(6) = 180.000000000000000d0
 phi_angular_integration_lebedev(7) = 45.000000000000000d0
 phi_angular_integration_lebedev(8) = 135.000000000000000d0
 phi_angular_integration_lebedev(9) = 45.000000000000000d0
 phi_angular_integration_lebedev(10) = 135.000000000000000d0
 phi_angular_integration_lebedev(11) = 45.000000000000000d0
 phi_angular_integration_lebedev(12) = 135.000000000000000d0
 phi_angular_integration_lebedev(13) = 45.000000000000000d0
 phi_angular_integration_lebedev(14) = 135.000000000000000d0
 phi_angular_integration_lebedev(15) = 90.000000000000000d0
 phi_angular_integration_lebedev(16) = 90.000000000000000d0
 phi_angular_integration_lebedev(17) = 90.000000000000000d0
 phi_angular_integration_lebedev(18) = 90.000000000000000d0
 phi_angular_integration_lebedev(19) = 54.735610317245346d0
 phi_angular_integration_lebedev(20) = 125.264389682754654d0
 phi_angular_integration_lebedev(21) = 54.735610317245346d0
 phi_angular_integration_lebedev(22) = 125.264389682754654d0
 phi_angular_integration_lebedev(23) = 54.735610317245346d0
 phi_angular_integration_lebedev(24) = 125.264389682754654d0
 phi_angular_integration_lebedev(25) = 54.735610317245346d0
 phi_angular_integration_lebedev(26) = 125.264389682754654d0
 phi_angular_integration_lebedev(27) = 25.239401820678911d0
 phi_angular_integration_lebedev(28) = 154.760598179321079d0
 phi_angular_integration_lebedev(29) = 25.239401820678911d0
 phi_angular_integration_lebedev(30) = 154.760598179321079d0
 phi_angular_integration_lebedev(31) = 25.239401820678911d0
 phi_angular_integration_lebedev(32) = 154.760598179321079d0
 phi_angular_integration_lebedev(33) = 25.239401820678911d0
 phi_angular_integration_lebedev(34) = 154.760598179321079d0
 phi_angular_integration_lebedev(35) = 72.451599386207704d0
 phi_angular_integration_lebedev(36) = 72.451599386207704d0
 phi_angular_integration_lebedev(37) = 107.548400613792296d0
 phi_angular_integration_lebedev(38) = 107.548400613792296d0
 phi_angular_integration_lebedev(39) = 72.451599386207704d0
 phi_angular_integration_lebedev(40) = 72.451599386207704d0
 phi_angular_integration_lebedev(41) = 107.548400613792296d0
 phi_angular_integration_lebedev(42) = 107.548400613792296d0
 phi_angular_integration_lebedev(43) = 72.451599386207704d0
 phi_angular_integration_lebedev(44) = 72.451599386207704d0
 phi_angular_integration_lebedev(45) = 107.548400613792296d0
 phi_angular_integration_lebedev(46) = 107.548400613792296d0
 phi_angular_integration_lebedev(47) = 72.451599386207704d0
 phi_angular_integration_lebedev(48) = 72.451599386207704d0
 phi_angular_integration_lebedev(49) = 107.548400613792296d0
 phi_angular_integration_lebedev(50) = 107.548400613792296d0

 weights_angular_integration_lebedev(1) = 0.012698412698413d0
 weights_angular_integration_lebedev(2) = 0.012698412698413d0
 weights_angular_integration_lebedev(3) = 0.012698412698413d0
 weights_angular_integration_lebedev(4) = 0.012698412698413d0
 weights_angular_integration_lebedev(5) = 0.012698412698413d0
 weights_angular_integration_lebedev(6) = 0.012698412698413d0
 weights_angular_integration_lebedev(7) = 0.022574955908289d0
 weights_angular_integration_lebedev(8) = 0.022574955908289d0
 weights_angular_integration_lebedev(9) = 0.022574955908289d0
 weights_angular_integration_lebedev(10) = 0.022574955908289d0
 weights_angular_integration_lebedev(11) = 0.022574955908289d0
 weights_angular_integration_lebedev(12) = 0.022574955908289d0
 weights_angular_integration_lebedev(13) = 0.022574955908289d0
 weights_angular_integration_lebedev(14) = 0.022574955908289d0
 weights_angular_integration_lebedev(15) = 0.022574955908289d0
 weights_angular_integration_lebedev(16) = 0.022574955908289d0
 weights_angular_integration_lebedev(17) = 0.022574955908289d0
 weights_angular_integration_lebedev(18) = 0.022574955908289d0
 weights_angular_integration_lebedev(19) = 0.021093750000000d0
 weights_angular_integration_lebedev(20) = 0.021093750000000d0
 weights_angular_integration_lebedev(21) = 0.021093750000000d0
 weights_angular_integration_lebedev(22) = 0.021093750000000d0
 weights_angular_integration_lebedev(23) = 0.021093750000000d0
 weights_angular_integration_lebedev(24) = 0.021093750000000d0
 weights_angular_integration_lebedev(25) = 0.021093750000000d0
 weights_angular_integration_lebedev(26) = 0.021093750000000d0
 weights_angular_integration_lebedev(27) = 0.020173335537919d0
 weights_angular_integration_lebedev(28) = 0.020173335537919d0
 weights_angular_integration_lebedev(29) = 0.020173335537919d0
 weights_angular_integration_lebedev(30) = 0.020173335537919d0
 weights_angular_integration_lebedev(31) = 0.020173335537919d0
 weights_angular_integration_lebedev(32) = 0.020173335537919d0
 weights_angular_integration_lebedev(33) = 0.020173335537919d0
 weights_angular_integration_lebedev(34) = 0.020173335537919d0
 weights_angular_integration_lebedev(35) = 0.020173335537919d0
 weights_angular_integration_lebedev(36) = 0.020173335537919d0
 weights_angular_integration_lebedev(37) = 0.020173335537919d0
 weights_angular_integration_lebedev(38) = 0.020173335537919d0
 weights_angular_integration_lebedev(39) = 0.020173335537919d0
 weights_angular_integration_lebedev(40) = 0.020173335537919d0
 weights_angular_integration_lebedev(41) = 0.020173335537919d0
 weights_angular_integration_lebedev(42) = 0.020173335537919d0
 weights_angular_integration_lebedev(43) = 0.020173335537919d0
 weights_angular_integration_lebedev(44) = 0.020173335537919d0
 weights_angular_integration_lebedev(45) = 0.020173335537919d0
 weights_angular_integration_lebedev(46) = 0.020173335537919d0
 weights_angular_integration_lebedev(47) = 0.020173335537919d0
 weights_angular_integration_lebedev(48) = 0.020173335537919d0
 weights_angular_integration_lebedev(49) = 0.020173335537919d0
 weights_angular_integration_lebedev(50) = 0.020173335537919d0

 else if (degree_max_integration_lebedev == 13)then


 theta_angular_integration_lebedev(1) = 0.000000000000000d0
 theta_angular_integration_lebedev(2) = 180.000000000000000d0
 theta_angular_integration_lebedev(3) = 90.000000000000000d0
 theta_angular_integration_lebedev(4) = -90.000000000000000d0
 theta_angular_integration_lebedev(5) = 90.000000000000000d0
 theta_angular_integration_lebedev(6) = 90.000000000000000d0
 theta_angular_integration_lebedev(7) = 90.000000000000000d0
 theta_angular_integration_lebedev(8) = 90.000000000000000d0
 theta_angular_integration_lebedev(9) = -90.000000000000000d0
 theta_angular_integration_lebedev(10) = -90.000000000000000d0
 theta_angular_integration_lebedev(11) = 0.000000000000000d0
 theta_angular_integration_lebedev(12) = 0.000000000000000d0
 theta_angular_integration_lebedev(13) = 180.000000000000000d0
 theta_angular_integration_lebedev(14) = 180.000000000000000d0
 theta_angular_integration_lebedev(15) = 45.000000000000000d0
 theta_angular_integration_lebedev(16) = -45.000000000000000d0
 theta_angular_integration_lebedev(17) = 135.000000000000000d0
 theta_angular_integration_lebedev(18) = -135.000000000000000d0
 theta_angular_integration_lebedev(19) = 45.000000000000000d0
 theta_angular_integration_lebedev(20) = 45.000000000000000d0
 theta_angular_integration_lebedev(21) = -45.000000000000000d0
 theta_angular_integration_lebedev(22) = -45.000000000000000d0
 theta_angular_integration_lebedev(23) = 135.000000000000000d0
 theta_angular_integration_lebedev(24) = 135.000000000000000d0
 theta_angular_integration_lebedev(25) = -135.000000000000000d0
 theta_angular_integration_lebedev(26) = -135.000000000000000d0
 theta_angular_integration_lebedev(27) = 45.000000000000000d0
 theta_angular_integration_lebedev(28) = 45.000000000000000d0
 theta_angular_integration_lebedev(29) = -45.000000000000000d0
 theta_angular_integration_lebedev(30) = -45.000000000000000d0
 theta_angular_integration_lebedev(31) = 135.000000000000000d0
 theta_angular_integration_lebedev(32) = 135.000000000000000d0
 theta_angular_integration_lebedev(33) = -135.000000000000000d0
 theta_angular_integration_lebedev(34) = -135.000000000000000d0
 theta_angular_integration_lebedev(35) = 56.789089239100910d0
 theta_angular_integration_lebedev(36) = -56.789089239100910d0
 theta_angular_integration_lebedev(37) = 56.789089239100910d0
 theta_angular_integration_lebedev(38) = -56.789089239100910d0
 theta_angular_integration_lebedev(39) = 123.210910760899083d0
 theta_angular_integration_lebedev(40) = -123.210910760899083d0
 theta_angular_integration_lebedev(41) = 123.210910760899083d0
 theta_angular_integration_lebedev(42) = -123.210910760899083d0
 theta_angular_integration_lebedev(43) = 33.210910760899083d0
 theta_angular_integration_lebedev(44) = 146.789089239100917d0
 theta_angular_integration_lebedev(45) = 33.210910760899083d0
 theta_angular_integration_lebedev(46) = 146.789089239100917d0
 theta_angular_integration_lebedev(47) = -33.210910760899083d0
 theta_angular_integration_lebedev(48) = -146.789089239100917d0
 theta_angular_integration_lebedev(49) = -33.210910760899083d0
 theta_angular_integration_lebedev(50) = -146.789089239100917d0
 theta_angular_integration_lebedev(51) = 71.290342151652581d0
 theta_angular_integration_lebedev(52) = -71.290342151652581d0
 theta_angular_integration_lebedev(53) = 108.709657848347433d0
 theta_angular_integration_lebedev(54) = -108.709657848347433d0
 theta_angular_integration_lebedev(55) = 18.709657848347415d0
 theta_angular_integration_lebedev(56) = -18.709657848347415d0
 theta_angular_integration_lebedev(57) = 161.290342151652595d0
 theta_angular_integration_lebedev(58) = -161.290342151652595d0
 theta_angular_integration_lebedev(59) = 0.000000000000000d0
 theta_angular_integration_lebedev(60) = 0.000000000000000d0
 theta_angular_integration_lebedev(61) = 180.000000000000000d0
 theta_angular_integration_lebedev(62) = 180.000000000000000d0
 theta_angular_integration_lebedev(63) = 0.000000000000000d0
 theta_angular_integration_lebedev(64) = 0.000000000000000d0
 theta_angular_integration_lebedev(65) = 180.000000000000000d0
 theta_angular_integration_lebedev(66) = 180.000000000000000d0
 theta_angular_integration_lebedev(67) = 90.000000000000000d0
 theta_angular_integration_lebedev(68) = 90.000000000000000d0
 theta_angular_integration_lebedev(69) = -90.000000000000000d0
 theta_angular_integration_lebedev(70) = -90.000000000000000d0
 theta_angular_integration_lebedev(71) = 90.000000000000000d0
 theta_angular_integration_lebedev(72) = 90.000000000000000d0
 theta_angular_integration_lebedev(73) = -90.000000000000000d0
 theta_angular_integration_lebedev(74) = -90.000000000000000d0

 phi_angular_integration_lebedev(1) = 90.000000000000000d0
 phi_angular_integration_lebedev(2) = 90.000000000000000d0
 phi_angular_integration_lebedev(3) = 90.000000000000000d0
 phi_angular_integration_lebedev(4) = 90.000000000000000d0
 phi_angular_integration_lebedev(5) = 0.000000000000000d0
 phi_angular_integration_lebedev(6) = 180.000000000000000d0
 phi_angular_integration_lebedev(7) = 45.000000000000000d0
 phi_angular_integration_lebedev(8) = 135.000000000000000d0
 phi_angular_integration_lebedev(9) = 45.000000000000000d0
 phi_angular_integration_lebedev(10) = 135.000000000000000d0
 phi_angular_integration_lebedev(11) = 45.000000000000000d0
 phi_angular_integration_lebedev(12) = 135.000000000000000d0
 phi_angular_integration_lebedev(13) = 45.000000000000000d0
 phi_angular_integration_lebedev(14) = 135.000000000000000d0
 phi_angular_integration_lebedev(15) = 90.000000000000000d0
 phi_angular_integration_lebedev(16) = 90.000000000000000d0
 phi_angular_integration_lebedev(17) = 90.000000000000000d0
 phi_angular_integration_lebedev(18) = 90.000000000000000d0
 phi_angular_integration_lebedev(19) = 54.735610317245346d0
 phi_angular_integration_lebedev(20) = 125.264389682754654d0
 phi_angular_integration_lebedev(21) = 54.735610317245346d0
 phi_angular_integration_lebedev(22) = 125.264389682754654d0
 phi_angular_integration_lebedev(23) = 54.735610317245346d0
 phi_angular_integration_lebedev(24) = 125.264389682754654d0
 phi_angular_integration_lebedev(25) = 54.735610317245346d0
 phi_angular_integration_lebedev(26) = 125.264389682754654d0
 phi_angular_integration_lebedev(27) = 42.794137107114921d0
 phi_angular_integration_lebedev(28) = 137.205862892885108d0
 phi_angular_integration_lebedev(29) = 42.794137107114921d0
 phi_angular_integration_lebedev(30) = 137.205862892885108d0
 phi_angular_integration_lebedev(31) = 42.794137107114921d0
 phi_angular_integration_lebedev(32) = 137.205862892885108d0
 phi_angular_integration_lebedev(33) = 42.794137107114921d0
 phi_angular_integration_lebedev(34) = 137.205862892885108d0
 phi_angular_integration_lebedev(35) = 61.289485196402069d0
 phi_angular_integration_lebedev(36) = 61.289485196402069d0
 phi_angular_integration_lebedev(37) = 118.710514803597945d0
 phi_angular_integration_lebedev(38) = 118.710514803597945d0
 phi_angular_integration_lebedev(39) = 61.289485196402069d0
 phi_angular_integration_lebedev(40) = 61.289485196402069d0
 phi_angular_integration_lebedev(41) = 118.710514803597945d0
 phi_angular_integration_lebedev(42) = 118.710514803597945d0
 phi_angular_integration_lebedev(43) = 61.289485196402069d0
 phi_angular_integration_lebedev(44) = 61.289485196402069d0
 phi_angular_integration_lebedev(45) = 118.710514803597945d0
 phi_angular_integration_lebedev(46) = 118.710514803597945d0
 phi_angular_integration_lebedev(47) = 61.289485196402069d0
 phi_angular_integration_lebedev(48) = 61.289485196402069d0
 phi_angular_integration_lebedev(49) = 118.710514803597945d0
 phi_angular_integration_lebedev(50) = 118.710514803597945d0
 phi_angular_integration_lebedev(51) = 90.000000000000000d0
 phi_angular_integration_lebedev(52) = 90.000000000000000d0
 phi_angular_integration_lebedev(53) = 90.000000000000000d0
 phi_angular_integration_lebedev(54) = 90.000000000000000d0
 phi_angular_integration_lebedev(55) = 90.000000000000000d0
 phi_angular_integration_lebedev(56) = 90.000000000000000d0
 phi_angular_integration_lebedev(57) = 90.000000000000000d0
 phi_angular_integration_lebedev(58) = 90.000000000000000d0
 phi_angular_integration_lebedev(59) = 18.709657848347415d0
 phi_angular_integration_lebedev(60) = 161.290342151652595d0
 phi_angular_integration_lebedev(61) = 18.709657848347415d0
 phi_angular_integration_lebedev(62) = 161.290342151652595d0
 phi_angular_integration_lebedev(63) = 71.290342151652581d0
 phi_angular_integration_lebedev(64) = 108.709657848347433d0
 phi_angular_integration_lebedev(65) = 71.290342151652581d0
 phi_angular_integration_lebedev(66) = 108.709657848347433d0
 phi_angular_integration_lebedev(67) = 18.709657848347415d0
 phi_angular_integration_lebedev(68) = 161.290342151652595d0
 phi_angular_integration_lebedev(69) = 18.709657848347415d0
 phi_angular_integration_lebedev(70) = 161.290342151652595d0
 phi_angular_integration_lebedev(71) = 71.290342151652581d0
 phi_angular_integration_lebedev(72) = 108.709657848347433d0
 phi_angular_integration_lebedev(73) = 71.290342151652581d0
 phi_angular_integration_lebedev(74) = 108.709657848347433d0

 weights_angular_integration_lebedev(1) = 0.000513067179734d0
 weights_angular_integration_lebedev(2) = 0.000513067179734d0
 weights_angular_integration_lebedev(3) = 0.000513067179734d0
 weights_angular_integration_lebedev(4) = 0.000513067179734d0
 weights_angular_integration_lebedev(5) = 0.000513067179734d0
 weights_angular_integration_lebedev(6) = 0.000513067179734d0
 weights_angular_integration_lebedev(7) = 0.016604069565742d0
 weights_angular_integration_lebedev(8) = 0.016604069565742d0
 weights_angular_integration_lebedev(9) = 0.016604069565742d0
 weights_angular_integration_lebedev(10) = 0.016604069565742d0
 weights_angular_integration_lebedev(11) = 0.016604069565742d0
 weights_angular_integration_lebedev(12) = 0.016604069565742d0
 weights_angular_integration_lebedev(13) = 0.016604069565742d0
 weights_angular_integration_lebedev(14) = 0.016604069565742d0
 weights_angular_integration_lebedev(15) = 0.016604069565742d0
 weights_angular_integration_lebedev(16) = 0.016604069565742d0
 weights_angular_integration_lebedev(17) = 0.016604069565742d0
 weights_angular_integration_lebedev(18) = 0.016604069565742d0
 weights_angular_integration_lebedev(19) = -0.029586038961039d0
 weights_angular_integration_lebedev(20) = -0.029586038961039d0
 weights_angular_integration_lebedev(21) = -0.029586038961039d0
 weights_angular_integration_lebedev(22) = -0.029586038961039d0
 weights_angular_integration_lebedev(23) = -0.029586038961039d0
 weights_angular_integration_lebedev(24) = -0.029586038961039d0
 weights_angular_integration_lebedev(25) = -0.029586038961039d0
 weights_angular_integration_lebedev(26) = -0.029586038961039d0
 weights_angular_integration_lebedev(27) = 0.026576207082159d0
 weights_angular_integration_lebedev(28) = 0.026576207082159d0
 weights_angular_integration_lebedev(29) = 0.026576207082159d0
 weights_angular_integration_lebedev(30) = 0.026576207082159d0
 weights_angular_integration_lebedev(31) = 0.026576207082159d0
 weights_angular_integration_lebedev(32) = 0.026576207082159d0
 weights_angular_integration_lebedev(33) = 0.026576207082159d0
 weights_angular_integration_lebedev(34) = 0.026576207082159d0
 weights_angular_integration_lebedev(35) = 0.026576207082159d0
 weights_angular_integration_lebedev(36) = 0.026576207082159d0
 weights_angular_integration_lebedev(37) = 0.026576207082159d0
 weights_angular_integration_lebedev(38) = 0.026576207082159d0
 weights_angular_integration_lebedev(39) = 0.026576207082159d0
 weights_angular_integration_lebedev(40) = 0.026576207082159d0
 weights_angular_integration_lebedev(41) = 0.026576207082159d0
 weights_angular_integration_lebedev(42) = 0.026576207082159d0
 weights_angular_integration_lebedev(43) = 0.026576207082159d0
 weights_angular_integration_lebedev(44) = 0.026576207082159d0
 weights_angular_integration_lebedev(45) = 0.026576207082159d0
 weights_angular_integration_lebedev(46) = 0.026576207082159d0
 weights_angular_integration_lebedev(47) = 0.026576207082159d0
 weights_angular_integration_lebedev(48) = 0.026576207082159d0
 weights_angular_integration_lebedev(49) = 0.026576207082159d0
 weights_angular_integration_lebedev(50) = 0.026576207082159d0
 weights_angular_integration_lebedev(51) = 0.016522170993716d0
 weights_angular_integration_lebedev(52) = 0.016522170993716d0
 weights_angular_integration_lebedev(53) = 0.016522170993716d0
 weights_angular_integration_lebedev(54) = 0.016522170993716d0
 weights_angular_integration_lebedev(55) = 0.016522170993716d0
 weights_angular_integration_lebedev(56) = 0.016522170993716d0
 weights_angular_integration_lebedev(57) = 0.016522170993716d0
 weights_angular_integration_lebedev(58) = 0.016522170993716d0
 weights_angular_integration_lebedev(59) = 0.016522170993716d0
 weights_angular_integration_lebedev(60) = 0.016522170993716d0
 weights_angular_integration_lebedev(61) = 0.016522170993716d0
 weights_angular_integration_lebedev(62) = 0.016522170993716d0
 weights_angular_integration_lebedev(63) = 0.016522170993716d0
 weights_angular_integration_lebedev(64) = 0.016522170993716d0
 weights_angular_integration_lebedev(65) = 0.016522170993716d0
 weights_angular_integration_lebedev(66) = 0.016522170993716d0
 weights_angular_integration_lebedev(67) = 0.016522170993716d0
 weights_angular_integration_lebedev(68) = 0.016522170993716d0
 weights_angular_integration_lebedev(69) = 0.016522170993716d0
 weights_angular_integration_lebedev(70) = 0.016522170993716d0
 weights_angular_integration_lebedev(71) = 0.016522170993716d0
 weights_angular_integration_lebedev(72) = 0.016522170993716d0
 weights_angular_integration_lebedev(73) = 0.016522170993716d0
 weights_angular_integration_lebedev(74) = 0.016522170993716d0

 else if (degree_max_integration_lebedev == 15)then

 theta_angular_integration_lebedev(1) = 0.000000000000000d0
 theta_angular_integration_lebedev(2) = 180.000000000000000d0
 theta_angular_integration_lebedev(3) = 90.000000000000000d0
 theta_angular_integration_lebedev(4) = -90.000000000000000d0
 theta_angular_integration_lebedev(5) = 90.000000000000000d0
 theta_angular_integration_lebedev(6) = 90.000000000000000d0
 theta_angular_integration_lebedev(7) = 45.000000000000000d0
 theta_angular_integration_lebedev(8) = 45.000000000000000d0
 theta_angular_integration_lebedev(9) = -45.000000000000000d0
 theta_angular_integration_lebedev(10) = -45.000000000000000d0
 theta_angular_integration_lebedev(11) = 135.000000000000000d0
 theta_angular_integration_lebedev(12) = 135.000000000000000d0
 theta_angular_integration_lebedev(13) = -135.000000000000000d0
 theta_angular_integration_lebedev(14) = -135.000000000000000d0
 theta_angular_integration_lebedev(15) = 45.000000000000000d0
 theta_angular_integration_lebedev(16) = 45.000000000000000d0
 theta_angular_integration_lebedev(17) = -45.000000000000000d0
 theta_angular_integration_lebedev(18) = -45.000000000000000d0
 theta_angular_integration_lebedev(19) = 135.000000000000000d0
 theta_angular_integration_lebedev(20) = 135.000000000000000d0
 theta_angular_integration_lebedev(21) = -135.000000000000000d0
 theta_angular_integration_lebedev(22) = -135.000000000000000d0
 theta_angular_integration_lebedev(23) = 66.561222392026522d0
 theta_angular_integration_lebedev(24) = -66.561222392026522d0
 theta_angular_integration_lebedev(25) = 66.561222392026522d0
 theta_angular_integration_lebedev(26) = -66.561222392026522d0
 theta_angular_integration_lebedev(27) = 113.438777607973492d0
 theta_angular_integration_lebedev(28) = -113.438777607973492d0
 theta_angular_integration_lebedev(29) = 113.438777607973492d0
 theta_angular_integration_lebedev(30) = -113.438777607973492d0
 theta_angular_integration_lebedev(31) = 23.438777607973478d0
 theta_angular_integration_lebedev(32) = 156.561222392026536d0
 theta_angular_integration_lebedev(33) = 23.438777607973478d0
 theta_angular_integration_lebedev(34) = 156.561222392026536d0
 theta_angular_integration_lebedev(35) = -23.438777607973478d0
 theta_angular_integration_lebedev(36) = -156.561222392026536d0
 theta_angular_integration_lebedev(37) = -23.438777607973478d0
 theta_angular_integration_lebedev(38) = -156.561222392026536d0
 theta_angular_integration_lebedev(39) = 45.000000000000000d0
 theta_angular_integration_lebedev(40) = 45.000000000000000d0
 theta_angular_integration_lebedev(41) = -45.000000000000000d0
 theta_angular_integration_lebedev(42) = -45.000000000000000d0
 theta_angular_integration_lebedev(43) = 135.000000000000000d0
 theta_angular_integration_lebedev(44) = 135.000000000000000d0
 theta_angular_integration_lebedev(45) = -135.000000000000000d0
 theta_angular_integration_lebedev(46) = -135.000000000000000d0
 theta_angular_integration_lebedev(47) = 15.231635441931418d0
 theta_angular_integration_lebedev(48) = -15.231635441931418d0
 theta_angular_integration_lebedev(49) = 15.231635441931418d0
 theta_angular_integration_lebedev(50) = -15.231635441931418d0
 theta_angular_integration_lebedev(51) = 164.768364558068583d0
 theta_angular_integration_lebedev(52) = -164.768364558068583d0
 theta_angular_integration_lebedev(53) = 164.768364558068583d0
 theta_angular_integration_lebedev(54) = -164.768364558068583d0
 theta_angular_integration_lebedev(55) = 74.768364558068569d0
 theta_angular_integration_lebedev(56) = 105.231635441931431d0
 theta_angular_integration_lebedev(57) = 74.768364558068569d0
 theta_angular_integration_lebedev(58) = 105.231635441931431d0
 theta_angular_integration_lebedev(59) = -74.768364558068569d0
 theta_angular_integration_lebedev(60) = -105.231635441931431d0
 theta_angular_integration_lebedev(61) = -74.768364558068569d0
 theta_angular_integration_lebedev(62) = -105.231635441931431d0
 theta_angular_integration_lebedev(63) = 68.022464238570777d0
 theta_angular_integration_lebedev(64) = -68.022464238570777d0
 theta_angular_integration_lebedev(65) = 111.977535761429237d0
 theta_angular_integration_lebedev(66) = -111.977535761429237d0
 theta_angular_integration_lebedev(67) = 21.977535761429227d0
 theta_angular_integration_lebedev(68) = -21.977535761429227d0
 theta_angular_integration_lebedev(69) = 158.022464238570763d0
 theta_angular_integration_lebedev(70) = -158.022464238570763d0
 theta_angular_integration_lebedev(71) = 0.000000000000000d0
 theta_angular_integration_lebedev(72) = 0.000000000000000d0
 theta_angular_integration_lebedev(73) = 180.000000000000000d0
 theta_angular_integration_lebedev(74) = 180.000000000000000d0
 theta_angular_integration_lebedev(75) = 0.000000000000000d0
 theta_angular_integration_lebedev(76) = 0.000000000000000d0
 theta_angular_integration_lebedev(77) = 180.000000000000000d0
 theta_angular_integration_lebedev(78) = 180.000000000000000d0
 theta_angular_integration_lebedev(79) = 90.000000000000000d0
 theta_angular_integration_lebedev(80) = 90.000000000000000d0
 theta_angular_integration_lebedev(81) = -90.000000000000000d0
 theta_angular_integration_lebedev(82) = -90.000000000000000d0
 theta_angular_integration_lebedev(83) = 90.000000000000000d0
 theta_angular_integration_lebedev(84) = 90.000000000000000d0
 theta_angular_integration_lebedev(85) = -90.000000000000000d0
 theta_angular_integration_lebedev(86) = -90.000000000000000d0

 phi_angular_integration_lebedev(1) = 90.000000000000000d0
 phi_angular_integration_lebedev(2) = 90.000000000000000d0
 phi_angular_integration_lebedev(3) = 90.000000000000000d0
 phi_angular_integration_lebedev(4) = 90.000000000000000d0
 phi_angular_integration_lebedev(5) = 0.000000000000000d0
 phi_angular_integration_lebedev(6) = 180.000000000000000d0
 phi_angular_integration_lebedev(7) = 54.735610317245346d0
 phi_angular_integration_lebedev(8) = 125.264389682754654d0
 phi_angular_integration_lebedev(9) = 54.735610317245346d0
 phi_angular_integration_lebedev(10) = 125.264389682754654d0
 phi_angular_integration_lebedev(11) = 54.735610317245346d0
 phi_angular_integration_lebedev(12) = 125.264389682754654d0
 phi_angular_integration_lebedev(13) = 54.735610317245346d0
 phi_angular_integration_lebedev(14) = 125.264389682754654d0
 phi_angular_integration_lebedev(15) = 31.513359490876244d0
 phi_angular_integration_lebedev(16) = 148.486640509123760d0
 phi_angular_integration_lebedev(17) = 31.513359490876244d0
 phi_angular_integration_lebedev(18) = 148.486640509123760d0
 phi_angular_integration_lebedev(19) = 31.513359490876244d0
 phi_angular_integration_lebedev(20) = 148.486640509123760d0
 phi_angular_integration_lebedev(21) = 31.513359490876244d0
 phi_angular_integration_lebedev(22) = 148.486640509123760d0
 phi_angular_integration_lebedev(23) = 68.308874114180426d0
 phi_angular_integration_lebedev(24) = 68.308874114180426d0
 phi_angular_integration_lebedev(25) = 111.691125885819588d0
 phi_angular_integration_lebedev(26) = 111.691125885819588d0
 phi_angular_integration_lebedev(27) = 68.308874114180426d0
 phi_angular_integration_lebedev(28) = 68.308874114180426d0
 phi_angular_integration_lebedev(29) = 111.691125885819588d0
 phi_angular_integration_lebedev(30) = 111.691125885819588d0
 phi_angular_integration_lebedev(31) = 68.308874114180426d0
 phi_angular_integration_lebedev(32) = 68.308874114180426d0
 phi_angular_integration_lebedev(33) = 111.691125885819588d0
 phi_angular_integration_lebedev(34) = 111.691125885819588d0
 phi_angular_integration_lebedev(35) = 68.308874114180426d0
 phi_angular_integration_lebedev(36) = 68.308874114180426d0
 phi_angular_integration_lebedev(37) = 111.691125885819588d0
 phi_angular_integration_lebedev(38) = 111.691125885819588d0
 phi_angular_integration_lebedev(39) = 79.101860732695940d0
 phi_angular_integration_lebedev(40) = 100.898139267304060d0
 phi_angular_integration_lebedev(41) = 79.101860732695940d0
 phi_angular_integration_lebedev(42) = 100.898139267304060d0
 phi_angular_integration_lebedev(43) = 79.101860732695940d0
 phi_angular_integration_lebedev(44) = 100.898139267304060d0
 phi_angular_integration_lebedev(45) = 79.101860732695940d0
 phi_angular_integration_lebedev(46) = 100.898139267304060d0
 phi_angular_integration_lebedev(47) = 46.024237785326378d0
 phi_angular_integration_lebedev(48) = 46.024237785326378d0
 phi_angular_integration_lebedev(49) = 133.975762214673637d0
 phi_angular_integration_lebedev(50) = 133.975762214673637d0
 phi_angular_integration_lebedev(51) = 46.024237785326378d0
 phi_angular_integration_lebedev(52) = 46.024237785326378d0
 phi_angular_integration_lebedev(53) = 133.975762214673637d0
 phi_angular_integration_lebedev(54) = 133.975762214673637d0
 phi_angular_integration_lebedev(55) = 46.024237785326378d0
 phi_angular_integration_lebedev(56) = 46.024237785326378d0
 phi_angular_integration_lebedev(57) = 133.975762214673637d0
 phi_angular_integration_lebedev(58) = 133.975762214673637d0
 phi_angular_integration_lebedev(59) = 46.024237785326378d0
 phi_angular_integration_lebedev(60) = 46.024237785326378d0
 phi_angular_integration_lebedev(61) = 133.975762214673637d0
 phi_angular_integration_lebedev(62) = 133.975762214673637d0
 phi_angular_integration_lebedev(63) = 90.000000000000000d0
 phi_angular_integration_lebedev(64) = 90.000000000000000d0
 phi_angular_integration_lebedev(65) = 90.000000000000000d0
 phi_angular_integration_lebedev(66) = 90.000000000000000d0
 phi_angular_integration_lebedev(67) = 90.000000000000000d0
 phi_angular_integration_lebedev(68) = 90.000000000000000d0
 phi_angular_integration_lebedev(69) = 90.000000000000000d0
 phi_angular_integration_lebedev(70) = 90.000000000000000d0
 phi_angular_integration_lebedev(71) = 21.977535761429227d0
 phi_angular_integration_lebedev(72) = 158.022464238570763d0
 phi_angular_integration_lebedev(73) = 21.977535761429227d0
 phi_angular_integration_lebedev(74) = 158.022464238570763d0
 phi_angular_integration_lebedev(75) = 68.022464238570777d0
 phi_angular_integration_lebedev(76) = 111.977535761429237d0
 phi_angular_integration_lebedev(77) = 68.022464238570777d0
 phi_angular_integration_lebedev(78) = 111.977535761429237d0
 phi_angular_integration_lebedev(79) = 21.977535761429227d0
 phi_angular_integration_lebedev(80) = 158.022464238570763d0
 phi_angular_integration_lebedev(81) = 21.977535761429227d0
 phi_angular_integration_lebedev(82) = 158.022464238570763d0
 phi_angular_integration_lebedev(83) = 68.022464238570777d0
 phi_angular_integration_lebedev(84) = 111.977535761429237d0
 phi_angular_integration_lebedev(85) = 68.022464238570777d0
 phi_angular_integration_lebedev(86) = 111.977535761429237d0

 weights_angular_integration_lebedev(1) = 0.011544011544012d0
 weights_angular_integration_lebedev(2) = 0.011544011544012d0
 weights_angular_integration_lebedev(3) = 0.011544011544012d0
 weights_angular_integration_lebedev(4) = 0.011544011544012d0
 weights_angular_integration_lebedev(5) = 0.011544011544012d0
 weights_angular_integration_lebedev(6) = 0.011544011544012d0
 weights_angular_integration_lebedev(7) = 0.011943909085856d0
 weights_angular_integration_lebedev(8) = 0.011943909085856d0
 weights_angular_integration_lebedev(9) = 0.011943909085856d0
 weights_angular_integration_lebedev(10) = 0.011943909085856d0
 weights_angular_integration_lebedev(11) = 0.011943909085856d0
 weights_angular_integration_lebedev(12) = 0.011943909085856d0
 weights_angular_integration_lebedev(13) = 0.011943909085856d0
 weights_angular_integration_lebedev(14) = 0.011943909085856d0
 weights_angular_integration_lebedev(15) = 0.011110555710603d0
 weights_angular_integration_lebedev(16) = 0.011110555710603d0
 weights_angular_integration_lebedev(17) = 0.011110555710603d0
 weights_angular_integration_lebedev(18) = 0.011110555710603d0
 weights_angular_integration_lebedev(19) = 0.011110555710603d0
 weights_angular_integration_lebedev(20) = 0.011110555710603d0
 weights_angular_integration_lebedev(21) = 0.011110555710603d0
 weights_angular_integration_lebedev(22) = 0.011110555710603d0
 weights_angular_integration_lebedev(23) = 0.011110555710603d0
 weights_angular_integration_lebedev(24) = 0.011110555710603d0
 weights_angular_integration_lebedev(25) = 0.011110555710603d0
 weights_angular_integration_lebedev(26) = 0.011110555710603d0
 weights_angular_integration_lebedev(27) = 0.011110555710603d0
 weights_angular_integration_lebedev(28) = 0.011110555710603d0
 weights_angular_integration_lebedev(29) = 0.011110555710603d0
 weights_angular_integration_lebedev(30) = 0.011110555710603d0
 weights_angular_integration_lebedev(31) = 0.011110555710603d0
 weights_angular_integration_lebedev(32) = 0.011110555710603d0
 weights_angular_integration_lebedev(33) = 0.011110555710603d0
 weights_angular_integration_lebedev(34) = 0.011110555710603d0
 weights_angular_integration_lebedev(35) = 0.011110555710603d0
 weights_angular_integration_lebedev(36) = 0.011110555710603d0
 weights_angular_integration_lebedev(37) = 0.011110555710603d0
 weights_angular_integration_lebedev(38) = 0.011110555710603d0
 weights_angular_integration_lebedev(39) = 0.011876501294537d0
 weights_angular_integration_lebedev(40) = 0.011876501294537d0
 weights_angular_integration_lebedev(41) = 0.011876501294537d0
 weights_angular_integration_lebedev(42) = 0.011876501294537d0
 weights_angular_integration_lebedev(43) = 0.011876501294537d0
 weights_angular_integration_lebedev(44) = 0.011876501294537d0
 weights_angular_integration_lebedev(45) = 0.011876501294537d0
 weights_angular_integration_lebedev(46) = 0.011876501294537d0
 weights_angular_integration_lebedev(47) = 0.011876501294537d0
 weights_angular_integration_lebedev(48) = 0.011876501294537d0
 weights_angular_integration_lebedev(49) = 0.011876501294537d0
 weights_angular_integration_lebedev(50) = 0.011876501294537d0
 weights_angular_integration_lebedev(51) = 0.011876501294537d0
 weights_angular_integration_lebedev(52) = 0.011876501294537d0
 weights_angular_integration_lebedev(53) = 0.011876501294537d0
 weights_angular_integration_lebedev(54) = 0.011876501294537d0
 weights_angular_integration_lebedev(55) = 0.011876501294537d0
 weights_angular_integration_lebedev(56) = 0.011876501294537d0
 weights_angular_integration_lebedev(57) = 0.011876501294537d0
 weights_angular_integration_lebedev(58) = 0.011876501294537d0
 weights_angular_integration_lebedev(59) = 0.011876501294537d0
 weights_angular_integration_lebedev(60) = 0.011876501294537d0
 weights_angular_integration_lebedev(61) = 0.011876501294537d0
 weights_angular_integration_lebedev(62) = 0.011876501294537d0
 weights_angular_integration_lebedev(63) = 0.011812303746904d0
 weights_angular_integration_lebedev(64) = 0.011812303746904d0
 weights_angular_integration_lebedev(65) = 0.011812303746904d0
 weights_angular_integration_lebedev(66) = 0.011812303746904d0
 weights_angular_integration_lebedev(67) = 0.011812303746904d0
 weights_angular_integration_lebedev(68) = 0.011812303746904d0
 weights_angular_integration_lebedev(69) = 0.011812303746904d0
 weights_angular_integration_lebedev(70) = 0.011812303746904d0
 weights_angular_integration_lebedev(71) = 0.011812303746904d0
 weights_angular_integration_lebedev(72) = 0.011812303746904d0
 weights_angular_integration_lebedev(73) = 0.011812303746904d0
 weights_angular_integration_lebedev(74) = 0.011812303746904d0
 weights_angular_integration_lebedev(75) = 0.011812303746904d0
 weights_angular_integration_lebedev(76) = 0.011812303746904d0
 weights_angular_integration_lebedev(77) = 0.011812303746904d0
 weights_angular_integration_lebedev(78) = 0.011812303746904d0
 weights_angular_integration_lebedev(79) = 0.011812303746904d0
 weights_angular_integration_lebedev(80) = 0.011812303746904d0
 weights_angular_integration_lebedev(81) = 0.011812303746904d0
 weights_angular_integration_lebedev(82) = 0.011812303746904d0
 weights_angular_integration_lebedev(83) = 0.011812303746904d0
 weights_angular_integration_lebedev(84) = 0.011812303746904d0
 weights_angular_integration_lebedev(85) = 0.011812303746904d0
 weights_angular_integration_lebedev(86) = 0.011812303746904d0

 else if (degree_max_integration_lebedev == 17)then


 theta_angular_integration_lebedev(1) = 0.000000000000000d0
 theta_angular_integration_lebedev(2) = 180.000000000000000d0
 theta_angular_integration_lebedev(3) = 90.000000000000000d0
 theta_angular_integration_lebedev(4) = -90.000000000000000d0
 theta_angular_integration_lebedev(5) = 90.000000000000000d0
 theta_angular_integration_lebedev(6) = 90.000000000000000d0
 theta_angular_integration_lebedev(7) = 45.000000000000000d0
 theta_angular_integration_lebedev(8) = 45.000000000000000d0
 theta_angular_integration_lebedev(9) = -45.000000000000000d0
 theta_angular_integration_lebedev(10) = -45.000000000000000d0
 theta_angular_integration_lebedev(11) = 135.000000000000000d0
 theta_angular_integration_lebedev(12) = 135.000000000000000d0
 theta_angular_integration_lebedev(13) = -135.000000000000000d0
 theta_angular_integration_lebedev(14) = -135.000000000000000d0
 theta_angular_integration_lebedev(15) = 45.000000000000000d0
 theta_angular_integration_lebedev(16) = 45.000000000000000d0
 theta_angular_integration_lebedev(17) = -45.000000000000000d0
 theta_angular_integration_lebedev(18) = -45.000000000000000d0
 theta_angular_integration_lebedev(19) = 135.000000000000000d0
 theta_angular_integration_lebedev(20) = 135.000000000000000d0
 theta_angular_integration_lebedev(21) = -135.000000000000000d0
 theta_angular_integration_lebedev(22) = -135.000000000000000d0
 theta_angular_integration_lebedev(23) = 79.142249366142039d0
 theta_angular_integration_lebedev(24) = -79.142249366142039d0
 theta_angular_integration_lebedev(25) = 79.142249366142039d0
 theta_angular_integration_lebedev(26) = -79.142249366142039d0
 theta_angular_integration_lebedev(27) = 100.857750633857975d0
 theta_angular_integration_lebedev(28) = -100.857750633857975d0
 theta_angular_integration_lebedev(29) = 100.857750633857975d0
 theta_angular_integration_lebedev(30) = -100.857750633857975d0
 theta_angular_integration_lebedev(31) = 10.857750633857979d0
 theta_angular_integration_lebedev(32) = 169.142249366142011d0
 theta_angular_integration_lebedev(33) = 10.857750633857979d0
 theta_angular_integration_lebedev(34) = 169.142249366142011d0
 theta_angular_integration_lebedev(35) = -10.857750633857979d0
 theta_angular_integration_lebedev(36) = -169.142249366142011d0
 theta_angular_integration_lebedev(37) = -10.857750633857979d0
 theta_angular_integration_lebedev(38) = -169.142249366142011d0
 theta_angular_integration_lebedev(39) = 45.000000000000000d0
 theta_angular_integration_lebedev(40) = 45.000000000000000d0
 theta_angular_integration_lebedev(41) = -45.000000000000000d0
 theta_angular_integration_lebedev(42) = -45.000000000000000d0
 theta_angular_integration_lebedev(43) = 135.000000000000000d0
 theta_angular_integration_lebedev(44) = 135.000000000000000d0
 theta_angular_integration_lebedev(45) = -135.000000000000000d0
 theta_angular_integration_lebedev(46) = -135.000000000000000d0
 theta_angular_integration_lebedev(47) = 17.369198090442442d0
 theta_angular_integration_lebedev(48) = -17.369198090442442d0
 theta_angular_integration_lebedev(49) = 17.369198090442442d0
 theta_angular_integration_lebedev(50) = -17.369198090442442d0
 theta_angular_integration_lebedev(51) = 162.630801909557562d0
 theta_angular_integration_lebedev(52) = -162.630801909557562d0
 theta_angular_integration_lebedev(53) = 162.630801909557562d0
 theta_angular_integration_lebedev(54) = -162.630801909557562d0
 theta_angular_integration_lebedev(55) = 72.630801909557576d0
 theta_angular_integration_lebedev(56) = 107.369198090442438d0
 theta_angular_integration_lebedev(57) = 72.630801909557576d0
 theta_angular_integration_lebedev(58) = 107.369198090442438d0
 theta_angular_integration_lebedev(59) = -72.630801909557576d0
 theta_angular_integration_lebedev(60) = -107.369198090442438d0
 theta_angular_integration_lebedev(61) = -72.630801909557576d0
 theta_angular_integration_lebedev(62) = -107.369198090442438d0
 theta_angular_integration_lebedev(63) = 45.000000000000000d0
 theta_angular_integration_lebedev(64) = 45.000000000000000d0
 theta_angular_integration_lebedev(65) = -45.000000000000000d0
 theta_angular_integration_lebedev(66) = -45.000000000000000d0
 theta_angular_integration_lebedev(67) = 135.000000000000000d0
 theta_angular_integration_lebedev(68) = 135.000000000000000d0
 theta_angular_integration_lebedev(69) = -135.000000000000000d0
 theta_angular_integration_lebedev(70) = -135.000000000000000d0
 theta_angular_integration_lebedev(71) = 64.478237843366074d0
 theta_angular_integration_lebedev(72) = -64.478237843366074d0
 theta_angular_integration_lebedev(73) = 64.478237843366074d0
 theta_angular_integration_lebedev(74) = -64.478237843366074d0
 theta_angular_integration_lebedev(75) = 115.521762156633926d0
 theta_angular_integration_lebedev(76) = -115.521762156633926d0
 theta_angular_integration_lebedev(77) = 115.521762156633926d0
 theta_angular_integration_lebedev(78) = -115.521762156633926d0
 theta_angular_integration_lebedev(79) = 25.521762156633940d0
 theta_angular_integration_lebedev(80) = 154.478237843366060d0
 theta_angular_integration_lebedev(81) = 25.521762156633940d0
 theta_angular_integration_lebedev(82) = 154.478237843366060d0
 theta_angular_integration_lebedev(83) = -25.521762156633940d0
 theta_angular_integration_lebedev(84) = -154.478237843366060d0
 theta_angular_integration_lebedev(85) = -25.521762156633940d0
 theta_angular_integration_lebedev(86) = -154.478237843366060d0
 theta_angular_integration_lebedev(87) = 61.421065207205885d0
 theta_angular_integration_lebedev(88) = -61.421065207205885d0
 theta_angular_integration_lebedev(89) = 118.578934792794129d0
 theta_angular_integration_lebedev(90) = -118.578934792794129d0
 theta_angular_integration_lebedev(91) = 28.578934792794119d0
 theta_angular_integration_lebedev(92) = -28.578934792794119d0
 theta_angular_integration_lebedev(93) = 151.421065207205885d0
 theta_angular_integration_lebedev(94) = -151.421065207205885d0
 theta_angular_integration_lebedev(95) = 0.000000000000000d0
 theta_angular_integration_lebedev(96) = 0.000000000000000d0
 theta_angular_integration_lebedev(97) = 180.000000000000000d0
 theta_angular_integration_lebedev(98) = 180.000000000000000d0
 theta_angular_integration_lebedev(99) = 0.000000000000000d0
 theta_angular_integration_lebedev(100) = 0.000000000000000d0
 theta_angular_integration_lebedev(101) = 180.000000000000000d0
 theta_angular_integration_lebedev(102) = 180.000000000000000d0
 theta_angular_integration_lebedev(103) = 90.000000000000000d0
 theta_angular_integration_lebedev(104) = 90.000000000000000d0
 theta_angular_integration_lebedev(105) = -90.000000000000000d0
 theta_angular_integration_lebedev(106) = -90.000000000000000d0
 theta_angular_integration_lebedev(107) = 90.000000000000000d0
 theta_angular_integration_lebedev(108) = 90.000000000000000d0
 theta_angular_integration_lebedev(109) = -90.000000000000000d0
 theta_angular_integration_lebedev(110) = -90.000000000000000d0

 phi_angular_integration_lebedev(1) = 90.000000000000000d0
 phi_angular_integration_lebedev(2) = 90.000000000000000d0
 phi_angular_integration_lebedev(3) = 90.000000000000000d0
 phi_angular_integration_lebedev(4) = 90.000000000000000d0
 phi_angular_integration_lebedev(5) = 0.000000000000000d0
 phi_angular_integration_lebedev(6) = 180.000000000000000d0
 phi_angular_integration_lebedev(7) = 54.735610317245346d0
 phi_angular_integration_lebedev(8) = 125.264389682754654d0
 phi_angular_integration_lebedev(9) = 54.735610317245346d0
 phi_angular_integration_lebedev(10) = 125.264389682754654d0
 phi_angular_integration_lebedev(11) = 54.735610317245346d0
 phi_angular_integration_lebedev(12) = 125.264389682754654d0
 phi_angular_integration_lebedev(13) = 54.735610317245346d0
 phi_angular_integration_lebedev(14) = 125.264389682754654d0
 phi_angular_integration_lebedev(15) = 15.176481556075425d0
 phi_angular_integration_lebedev(16) = 164.823518443924598d0
 phi_angular_integration_lebedev(17) = 15.176481556075425d0
 phi_angular_integration_lebedev(18) = 164.823518443924598d0
 phi_angular_integration_lebedev(19) = 15.176481556075425d0
 phi_angular_integration_lebedev(20) = 164.823518443924598d0
 phi_angular_integration_lebedev(21) = 15.176481556075425d0
 phi_angular_integration_lebedev(22) = 164.823518443924598d0
 phi_angular_integration_lebedev(23) = 79.332125737120890d0
 phi_angular_integration_lebedev(24) = 79.332125737120890d0
 phi_angular_integration_lebedev(25) = 100.667874262879110d0
 phi_angular_integration_lebedev(26) = 100.667874262879110d0
 phi_angular_integration_lebedev(27) = 79.332125737120890d0
 phi_angular_integration_lebedev(28) = 79.332125737120890d0
 phi_angular_integration_lebedev(29) = 100.667874262879110d0
 phi_angular_integration_lebedev(30) = 100.667874262879110d0
 phi_angular_integration_lebedev(31) = 79.332125737120890d0
 phi_angular_integration_lebedev(32) = 79.332125737120890d0
 phi_angular_integration_lebedev(33) = 100.667874262879110d0
 phi_angular_integration_lebedev(34) = 100.667874262879110d0
 phi_angular_integration_lebedev(35) = 79.332125737120890d0
 phi_angular_integration_lebedev(36) = 79.332125737120890d0
 phi_angular_integration_lebedev(37) = 100.667874262879110d0
 phi_angular_integration_lebedev(38) = 100.667874262879110d0
 phi_angular_integration_lebedev(39) = 77.528304436344783d0
 phi_angular_integration_lebedev(40) = 102.471695563655231d0
 phi_angular_integration_lebedev(41) = 77.528304436344783d0
 phi_angular_integration_lebedev(42) = 102.471695563655231d0
 phi_angular_integration_lebedev(43) = 77.528304436344783d0
 phi_angular_integration_lebedev(44) = 102.471695563655231d0
 phi_angular_integration_lebedev(45) = 77.528304436344783d0
 phi_angular_integration_lebedev(46) = 102.471695563655231d0
 phi_angular_integration_lebedev(47) = 46.336552258147378d0
 phi_angular_integration_lebedev(48) = 46.336552258147378d0
 phi_angular_integration_lebedev(49) = 133.663447741852622d0
 phi_angular_integration_lebedev(50) = 133.663447741852622d0
 phi_angular_integration_lebedev(51) = 46.336552258147378d0
 phi_angular_integration_lebedev(52) = 46.336552258147378d0
 phi_angular_integration_lebedev(53) = 133.663447741852622d0
 phi_angular_integration_lebedev(54) = 133.663447741852622d0
 phi_angular_integration_lebedev(55) = 46.336552258147378d0
 phi_angular_integration_lebedev(56) = 46.336552258147378d0
 phi_angular_integration_lebedev(57) = 133.663447741852622d0
 phi_angular_integration_lebedev(58) = 133.663447741852622d0
 phi_angular_integration_lebedev(59) = 46.336552258147378d0
 phi_angular_integration_lebedev(60) = 46.336552258147378d0
 phi_angular_integration_lebedev(61) = 133.663447741852622d0
 phi_angular_integration_lebedev(62) = 133.663447741852622d0
 phi_angular_integration_lebedev(63) = 34.027408105510005d0
 phi_angular_integration_lebedev(64) = 145.972591894489995d0
 phi_angular_integration_lebedev(65) = 34.027408105510005d0
 phi_angular_integration_lebedev(66) = 145.972591894489995d0
 phi_angular_integration_lebedev(67) = 34.027408105510005d0
 phi_angular_integration_lebedev(68) = 145.972591894489995d0
 phi_angular_integration_lebedev(69) = 34.027408105510005d0
 phi_angular_integration_lebedev(70) = 145.972591894489995d0
 phi_angular_integration_lebedev(71) = 66.691018332392488d0
 phi_angular_integration_lebedev(72) = 66.691018332392488d0
 phi_angular_integration_lebedev(73) = 113.308981667607512d0
 phi_angular_integration_lebedev(74) = 113.308981667607512d0
 phi_angular_integration_lebedev(75) = 66.691018332392488d0
 phi_angular_integration_lebedev(76) = 66.691018332392488d0
 phi_angular_integration_lebedev(77) = 113.308981667607512d0
 phi_angular_integration_lebedev(78) = 113.308981667607512d0
 phi_angular_integration_lebedev(79) = 66.691018332392488d0
 phi_angular_integration_lebedev(80) = 66.691018332392488d0
 phi_angular_integration_lebedev(81) = 113.308981667607512d0
 phi_angular_integration_lebedev(82) = 113.308981667607512d0
 phi_angular_integration_lebedev(83) = 66.691018332392488d0
 phi_angular_integration_lebedev(84) = 66.691018332392488d0
 phi_angular_integration_lebedev(85) = 113.308981667607512d0
 phi_angular_integration_lebedev(86) = 113.308981667607512d0
 phi_angular_integration_lebedev(87) = 90.000000000000000d0
 phi_angular_integration_lebedev(88) = 90.000000000000000d0
 phi_angular_integration_lebedev(89) = 90.000000000000000d0
 phi_angular_integration_lebedev(90) = 90.000000000000000d0
 phi_angular_integration_lebedev(91) = 90.000000000000000d0
 phi_angular_integration_lebedev(92) = 90.000000000000000d0
 phi_angular_integration_lebedev(93) = 90.000000000000000d0
 phi_angular_integration_lebedev(94) = 90.000000000000000d0
 phi_angular_integration_lebedev(95) = 28.578934792794119d0
 phi_angular_integration_lebedev(96) = 151.421065207205885d0
 phi_angular_integration_lebedev(97) = 28.578934792794119d0
 phi_angular_integration_lebedev(98) = 151.421065207205885d0
 phi_angular_integration_lebedev(99) = 61.421065207205885d0
 phi_angular_integration_lebedev(100) = 118.578934792794129d0
 phi_angular_integration_lebedev(101) = 61.421065207205885d0
 phi_angular_integration_lebedev(102) = 118.578934792794129d0
 phi_angular_integration_lebedev(103) = 28.578934792794119d0
 phi_angular_integration_lebedev(104) = 151.421065207205885d0
 phi_angular_integration_lebedev(105) = 28.578934792794119d0
 phi_angular_integration_lebedev(106) = 151.421065207205885d0
 phi_angular_integration_lebedev(107) = 61.421065207205885d0
 phi_angular_integration_lebedev(108) = 118.578934792794129d0
 phi_angular_integration_lebedev(109) = 61.421065207205885d0
 phi_angular_integration_lebedev(110) = 118.578934792794129d0

 weights_angular_integration_lebedev(1) = 0.003828270494937d0
 weights_angular_integration_lebedev(2) = 0.003828270494937d0
 weights_angular_integration_lebedev(3) = 0.003828270494937d0
 weights_angular_integration_lebedev(4) = 0.003828270494937d0
 weights_angular_integration_lebedev(5) = 0.003828270494937d0
 weights_angular_integration_lebedev(6) = 0.003828270494937d0
 weights_angular_integration_lebedev(7) = 0.009793737512488d0
 weights_angular_integration_lebedev(8) = 0.009793737512488d0
 weights_angular_integration_lebedev(9) = 0.009793737512488d0
 weights_angular_integration_lebedev(10) = 0.009793737512488d0
 weights_angular_integration_lebedev(11) = 0.009793737512488d0
 weights_angular_integration_lebedev(12) = 0.009793737512488d0
 weights_angular_integration_lebedev(13) = 0.009793737512488d0
 weights_angular_integration_lebedev(14) = 0.009793737512488d0
 weights_angular_integration_lebedev(15) = 0.008211737283191d0
 weights_angular_integration_lebedev(16) = 0.008211737283191d0
 weights_angular_integration_lebedev(17) = 0.008211737283191d0
 weights_angular_integration_lebedev(18) = 0.008211737283191d0
 weights_angular_integration_lebedev(19) = 0.008211737283191d0
 weights_angular_integration_lebedev(20) = 0.008211737283191d0
 weights_angular_integration_lebedev(21) = 0.008211737283191d0
 weights_angular_integration_lebedev(22) = 0.008211737283191d0
 weights_angular_integration_lebedev(23) = 0.008211737283191d0
 weights_angular_integration_lebedev(24) = 0.008211737283191d0
 weights_angular_integration_lebedev(25) = 0.008211737283191d0
 weights_angular_integration_lebedev(26) = 0.008211737283191d0
 weights_angular_integration_lebedev(27) = 0.008211737283191d0
 weights_angular_integration_lebedev(28) = 0.008211737283191d0
 weights_angular_integration_lebedev(29) = 0.008211737283191d0
 weights_angular_integration_lebedev(30) = 0.008211737283191d0
 weights_angular_integration_lebedev(31) = 0.008211737283191d0
 weights_angular_integration_lebedev(32) = 0.008211737283191d0
 weights_angular_integration_lebedev(33) = 0.008211737283191d0
 weights_angular_integration_lebedev(34) = 0.008211737283191d0
 weights_angular_integration_lebedev(35) = 0.008211737283191d0
 weights_angular_integration_lebedev(36) = 0.008211737283191d0
 weights_angular_integration_lebedev(37) = 0.008211737283191d0
 weights_angular_integration_lebedev(38) = 0.008211737283191d0
 weights_angular_integration_lebedev(39) = 0.009942814891178d0
 weights_angular_integration_lebedev(40) = 0.009942814891178d0
 weights_angular_integration_lebedev(41) = 0.009942814891178d0
 weights_angular_integration_lebedev(42) = 0.009942814891178d0
 weights_angular_integration_lebedev(43) = 0.009942814891178d0
 weights_angular_integration_lebedev(44) = 0.009942814891178d0
 weights_angular_integration_lebedev(45) = 0.009942814891178d0
 weights_angular_integration_lebedev(46) = 0.009942814891178d0
 weights_angular_integration_lebedev(47) = 0.009942814891178d0
 weights_angular_integration_lebedev(48) = 0.009942814891178d0
 weights_angular_integration_lebedev(49) = 0.009942814891178d0
 weights_angular_integration_lebedev(50) = 0.009942814891178d0
 weights_angular_integration_lebedev(51) = 0.009942814891178d0
 weights_angular_integration_lebedev(52) = 0.009942814891178d0
 weights_angular_integration_lebedev(53) = 0.009942814891178d0
 weights_angular_integration_lebedev(54) = 0.009942814891178d0
 weights_angular_integration_lebedev(55) = 0.009942814891178d0
 weights_angular_integration_lebedev(56) = 0.009942814891178d0
 weights_angular_integration_lebedev(57) = 0.009942814891178d0
 weights_angular_integration_lebedev(58) = 0.009942814891178d0
 weights_angular_integration_lebedev(59) = 0.009942814891178d0
 weights_angular_integration_lebedev(60) = 0.009942814891178d0
 weights_angular_integration_lebedev(61) = 0.009942814891178d0
 weights_angular_integration_lebedev(62) = 0.009942814891178d0
 weights_angular_integration_lebedev(63) = 0.009595471336071d0
 weights_angular_integration_lebedev(64) = 0.009595471336071d0
 weights_angular_integration_lebedev(65) = 0.009595471336071d0
 weights_angular_integration_lebedev(66) = 0.009595471336071d0
 weights_angular_integration_lebedev(67) = 0.009595471336071d0
 weights_angular_integration_lebedev(68) = 0.009595471336071d0
 weights_angular_integration_lebedev(69) = 0.009595471336071d0
 weights_angular_integration_lebedev(70) = 0.009595471336071d0
 weights_angular_integration_lebedev(71) = 0.009595471336071d0
 weights_angular_integration_lebedev(72) = 0.009595471336071d0
 weights_angular_integration_lebedev(73) = 0.009595471336071d0
 weights_angular_integration_lebedev(74) = 0.009595471336071d0
 weights_angular_integration_lebedev(75) = 0.009595471336071d0
 weights_angular_integration_lebedev(76) = 0.009595471336071d0
 weights_angular_integration_lebedev(77) = 0.009595471336071d0
 weights_angular_integration_lebedev(78) = 0.009595471336071d0
 weights_angular_integration_lebedev(79) = 0.009595471336071d0
 weights_angular_integration_lebedev(80) = 0.009595471336071d0
 weights_angular_integration_lebedev(81) = 0.009595471336071d0
 weights_angular_integration_lebedev(82) = 0.009595471336071d0
 weights_angular_integration_lebedev(83) = 0.009595471336071d0
 weights_angular_integration_lebedev(84) = 0.009595471336071d0
 weights_angular_integration_lebedev(85) = 0.009595471336071d0
 weights_angular_integration_lebedev(86) = 0.009595471336071d0
 weights_angular_integration_lebedev(87) = 0.009694996361663d0
 weights_angular_integration_lebedev(88) = 0.009694996361663d0
 weights_angular_integration_lebedev(89) = 0.009694996361663d0
 weights_angular_integration_lebedev(90) = 0.009694996361663d0
 weights_angular_integration_lebedev(91) = 0.009694996361663d0
 weights_angular_integration_lebedev(92) = 0.009694996361663d0
 weights_angular_integration_lebedev(93) = 0.009694996361663d0
 weights_angular_integration_lebedev(94) = 0.009694996361663d0
 weights_angular_integration_lebedev(95) = 0.009694996361663d0
 weights_angular_integration_lebedev(96) = 0.009694996361663d0
 weights_angular_integration_lebedev(97) = 0.009694996361663d0
 weights_angular_integration_lebedev(98) = 0.009694996361663d0
 weights_angular_integration_lebedev(99) = 0.009694996361663d0
 weights_angular_integration_lebedev(100) = 0.009694996361663d0
 weights_angular_integration_lebedev(101) = 0.009694996361663d0
 weights_angular_integration_lebedev(102) = 0.009694996361663d0
 weights_angular_integration_lebedev(103) = 0.009694996361663d0
 weights_angular_integration_lebedev(104) = 0.009694996361663d0
 weights_angular_integration_lebedev(105) = 0.009694996361663d0
 weights_angular_integration_lebedev(106) = 0.009694996361663d0
 weights_angular_integration_lebedev(107) = 0.009694996361663d0
 weights_angular_integration_lebedev(108) = 0.009694996361663d0
 weights_angular_integration_lebedev(109) = 0.009694996361663d0
 weights_angular_integration_lebedev(110) = 0.009694996361663d0

 else if (degree_max_integration_lebedev == 19)then


 theta_angular_integration_lebedev(1) = 0.000000000000000d0
 theta_angular_integration_lebedev(2) = 180.000000000000000d0
 theta_angular_integration_lebedev(3) = 90.000000000000000d0
 theta_angular_integration_lebedev(4) = -90.000000000000000d0
 theta_angular_integration_lebedev(5) = 90.000000000000000d0
 theta_angular_integration_lebedev(6) = 90.000000000000000d0
 theta_angular_integration_lebedev(7) = 90.000000000000000d0
 theta_angular_integration_lebedev(8) = 90.000000000000000d0
 theta_angular_integration_lebedev(9) = -90.000000000000000d0
 theta_angular_integration_lebedev(10) = -90.000000000000000d0
 theta_angular_integration_lebedev(11) = 0.000000000000000d0
 theta_angular_integration_lebedev(12) = 0.000000000000000d0
 theta_angular_integration_lebedev(13) = 180.000000000000000d0
 theta_angular_integration_lebedev(14) = 180.000000000000000d0
 theta_angular_integration_lebedev(15) = 45.000000000000000d0
 theta_angular_integration_lebedev(16) = -45.000000000000000d0
 theta_angular_integration_lebedev(17) = 135.000000000000000d0
 theta_angular_integration_lebedev(18) = -135.000000000000000d0
 theta_angular_integration_lebedev(19) = 45.000000000000000d0
 theta_angular_integration_lebedev(20) = 45.000000000000000d0
 theta_angular_integration_lebedev(21) = -45.000000000000000d0
 theta_angular_integration_lebedev(22) = -45.000000000000000d0
 theta_angular_integration_lebedev(23) = 135.000000000000000d0
 theta_angular_integration_lebedev(24) = 135.000000000000000d0
 theta_angular_integration_lebedev(25) = -135.000000000000000d0
 theta_angular_integration_lebedev(26) = -135.000000000000000d0
 theta_angular_integration_lebedev(27) = 45.000000000000000d0
 theta_angular_integration_lebedev(28) = 45.000000000000000d0
 theta_angular_integration_lebedev(29) = -45.000000000000000d0
 theta_angular_integration_lebedev(30) = -45.000000000000000d0
 theta_angular_integration_lebedev(31) = 135.000000000000000d0
 theta_angular_integration_lebedev(32) = 135.000000000000000d0
 theta_angular_integration_lebedev(33) = -135.000000000000000d0
 theta_angular_integration_lebedev(34) = -135.000000000000000d0
 theta_angular_integration_lebedev(35) = 23.298381329142529d0
 theta_angular_integration_lebedev(36) = -23.298381329142529d0
 theta_angular_integration_lebedev(37) = 23.298381329142529d0
 theta_angular_integration_lebedev(38) = -23.298381329142529d0
 theta_angular_integration_lebedev(39) = 156.701618670857499d0
 theta_angular_integration_lebedev(40) = -156.701618670857499d0
 theta_angular_integration_lebedev(41) = 156.701618670857499d0
 theta_angular_integration_lebedev(42) = -156.701618670857499d0
 theta_angular_integration_lebedev(43) = 66.701618670857471d0
 theta_angular_integration_lebedev(44) = 113.298381329142543d0
 theta_angular_integration_lebedev(45) = 66.701618670857471d0
 theta_angular_integration_lebedev(46) = 113.298381329142543d0
 theta_angular_integration_lebedev(47) = -66.701618670857471d0
 theta_angular_integration_lebedev(48) = -113.298381329142543d0
 theta_angular_integration_lebedev(49) = -66.701618670857471d0
 theta_angular_integration_lebedev(50) = -113.298381329142543d0
 theta_angular_integration_lebedev(51) = 45.000000000000000d0
 theta_angular_integration_lebedev(52) = 45.000000000000000d0
 theta_angular_integration_lebedev(53) = -45.000000000000000d0
 theta_angular_integration_lebedev(54) = -45.000000000000000d0
 theta_angular_integration_lebedev(55) = 135.000000000000000d0
 theta_angular_integration_lebedev(56) = 135.000000000000000d0
 theta_angular_integration_lebedev(57) = -135.000000000000000d0
 theta_angular_integration_lebedev(58) = -135.000000000000000d0
 theta_angular_integration_lebedev(59) = 62.648115190442049d0
 theta_angular_integration_lebedev(60) = -62.648115190442049d0
 theta_angular_integration_lebedev(61) = 62.648115190442049d0
 theta_angular_integration_lebedev(62) = -62.648115190442049d0
 theta_angular_integration_lebedev(63) = 117.351884809557959d0
 theta_angular_integration_lebedev(64) = -117.351884809557959d0
 theta_angular_integration_lebedev(65) = 117.351884809557959d0
 theta_angular_integration_lebedev(66) = -117.351884809557959d0
 theta_angular_integration_lebedev(67) = 27.351884809557951d0
 theta_angular_integration_lebedev(68) = 152.648115190442041d0
 theta_angular_integration_lebedev(69) = 27.351884809557951d0
 theta_angular_integration_lebedev(70) = 152.648115190442041d0
 theta_angular_integration_lebedev(71) = -27.351884809557951d0
 theta_angular_integration_lebedev(72) = -152.648115190442041d0
 theta_angular_integration_lebedev(73) = -27.351884809557951d0
 theta_angular_integration_lebedev(74) = -152.648115190442041d0
 theta_angular_integration_lebedev(75) = 45.000000000000000d0
 theta_angular_integration_lebedev(76) = 45.000000000000000d0
 theta_angular_integration_lebedev(77) = -45.000000000000000d0
 theta_angular_integration_lebedev(78) = -45.000000000000000d0
 theta_angular_integration_lebedev(79) = 135.000000000000000d0
 theta_angular_integration_lebedev(80) = 135.000000000000000d0
 theta_angular_integration_lebedev(81) = -135.000000000000000d0
 theta_angular_integration_lebedev(82) = -135.000000000000000d0
 theta_angular_integration_lebedev(83) = 80.824618269421777d0
 theta_angular_integration_lebedev(84) = -80.824618269421777d0
 theta_angular_integration_lebedev(85) = 80.824618269421777d0
 theta_angular_integration_lebedev(86) = -80.824618269421777d0
 theta_angular_integration_lebedev(87) = 99.175381730578238d0
 theta_angular_integration_lebedev(88) = -99.175381730578238d0
 theta_angular_integration_lebedev(89) = 99.175381730578238d0
 theta_angular_integration_lebedev(90) = -99.175381730578238d0
 theta_angular_integration_lebedev(91) = 9.175381730578227d0
 theta_angular_integration_lebedev(92) = 170.824618269421791d0
 theta_angular_integration_lebedev(93) = 9.175381730578227d0
 theta_angular_integration_lebedev(94) = 170.824618269421791d0
 theta_angular_integration_lebedev(95) = -9.175381730578227d0
 theta_angular_integration_lebedev(96) = -170.824618269421791d0
 theta_angular_integration_lebedev(97) = -9.175381730578227d0
 theta_angular_integration_lebedev(98) = -170.824618269421791d0
 theta_angular_integration_lebedev(99) = 72.653076890754207d0
 theta_angular_integration_lebedev(100) = 72.653076890754207d0
 theta_angular_integration_lebedev(101) = -72.653076890754207d0
 theta_angular_integration_lebedev(102) = -72.653076890754207d0
 theta_angular_integration_lebedev(103) = 107.346923109245793d0
 theta_angular_integration_lebedev(104) = 107.346923109245793d0
 theta_angular_integration_lebedev(105) = -107.346923109245793d0
 theta_angular_integration_lebedev(106) = -107.346923109245793d0
 theta_angular_integration_lebedev(107) = 80.960881834202326d0
 theta_angular_integration_lebedev(108) = 80.960881834202326d0
 theta_angular_integration_lebedev(109) = -80.960881834202326d0
 theta_angular_integration_lebedev(110) = -80.960881834202326d0
 theta_angular_integration_lebedev(111) = 99.039118165797674d0
 theta_angular_integration_lebedev(112) = 99.039118165797674d0
 theta_angular_integration_lebedev(113) = -99.039118165797674d0
 theta_angular_integration_lebedev(114) = -99.039118165797674d0
 theta_angular_integration_lebedev(115) = 17.346923109245800d0
 theta_angular_integration_lebedev(116) = 17.346923109245800d0
 theta_angular_integration_lebedev(117) = -17.346923109245800d0
 theta_angular_integration_lebedev(118) = -17.346923109245800d0
 theta_angular_integration_lebedev(119) = 162.653076890754193d0
 theta_angular_integration_lebedev(120) = 162.653076890754193d0
 theta_angular_integration_lebedev(121) = -162.653076890754193d0
 theta_angular_integration_lebedev(122) = -162.653076890754193d0
 theta_angular_integration_lebedev(123) = 63.010631515857654d0
 theta_angular_integration_lebedev(124) = 63.010631515857654d0
 theta_angular_integration_lebedev(125) = -63.010631515857654d0
 theta_angular_integration_lebedev(126) = -63.010631515857654d0
 theta_angular_integration_lebedev(127) = 116.989368484142346d0
 theta_angular_integration_lebedev(128) = 116.989368484142346d0
 theta_angular_integration_lebedev(129) = -116.989368484142346d0
 theta_angular_integration_lebedev(130) = -116.989368484142346d0
 theta_angular_integration_lebedev(131) = 9.039118165797701d0
 theta_angular_integration_lebedev(132) = 9.039118165797701d0
 theta_angular_integration_lebedev(133) = -9.039118165797701d0
 theta_angular_integration_lebedev(134) = -9.039118165797701d0
 theta_angular_integration_lebedev(135) = 170.960881834202297d0
 theta_angular_integration_lebedev(136) = 170.960881834202297d0
 theta_angular_integration_lebedev(137) = -170.960881834202297d0
 theta_angular_integration_lebedev(138) = -170.960881834202297d0
 theta_angular_integration_lebedev(139) = 26.989368484142357d0
 theta_angular_integration_lebedev(140) = 26.989368484142357d0
 theta_angular_integration_lebedev(141) = -26.989368484142357d0
 theta_angular_integration_lebedev(142) = -26.989368484142357d0
 theta_angular_integration_lebedev(143) = 153.010631515857654d0
 theta_angular_integration_lebedev(144) = 153.010631515857654d0
 theta_angular_integration_lebedev(145) = -153.010631515857654d0
 theta_angular_integration_lebedev(146) = -153.010631515857654d0

 phi_angular_integration_lebedev(1) = 90.000000000000000d0
 phi_angular_integration_lebedev(2) = 90.000000000000000d0
 phi_angular_integration_lebedev(3) = 90.000000000000000d0
 phi_angular_integration_lebedev(4) = 90.000000000000000d0
 phi_angular_integration_lebedev(5) = 0.000000000000000d0
 phi_angular_integration_lebedev(6) = 180.000000000000000d0
 phi_angular_integration_lebedev(7) = 45.000000000000000d0
 phi_angular_integration_lebedev(8) = 135.000000000000000d0
 phi_angular_integration_lebedev(9) = 45.000000000000000d0
 phi_angular_integration_lebedev(10) = 135.000000000000000d0
 phi_angular_integration_lebedev(11) = 45.000000000000000d0
 phi_angular_integration_lebedev(12) = 135.000000000000000d0
 phi_angular_integration_lebedev(13) = 45.000000000000000d0
 phi_angular_integration_lebedev(14) = 135.000000000000000d0
 phi_angular_integration_lebedev(15) = 90.000000000000000d0
 phi_angular_integration_lebedev(16) = 90.000000000000000d0
 phi_angular_integration_lebedev(17) = 90.000000000000000d0
 phi_angular_integration_lebedev(18) = 90.000000000000000d0
 phi_angular_integration_lebedev(19) = 54.735610317245346d0
 phi_angular_integration_lebedev(20) = 125.264389682754654d0
 phi_angular_integration_lebedev(21) = 54.735610317245346d0
 phi_angular_integration_lebedev(22) = 125.264389682754654d0
 phi_angular_integration_lebedev(23) = 54.735610317245346d0
 phi_angular_integration_lebedev(24) = 125.264389682754654d0
 phi_angular_integration_lebedev(25) = 54.735610317245346d0
 phi_angular_integration_lebedev(26) = 125.264389682754654d0
 phi_angular_integration_lebedev(27) = 73.064265786677382d0
 phi_angular_integration_lebedev(28) = 106.935734213322618d0
 phi_angular_integration_lebedev(29) = 73.064265786677382d0
 phi_angular_integration_lebedev(30) = 106.935734213322618d0
 phi_angular_integration_lebedev(31) = 73.064265786677382d0
 phi_angular_integration_lebedev(32) = 106.935734213322618d0
 phi_angular_integration_lebedev(33) = 73.064265786677382d0
 phi_angular_integration_lebedev(34) = 106.935734213322618d0
 phi_angular_integration_lebedev(35) = 47.433844543612018d0
 phi_angular_integration_lebedev(36) = 47.433844543612018d0
 phi_angular_integration_lebedev(37) = 132.566155456387975d0
 phi_angular_integration_lebedev(38) = 132.566155456387975d0
 phi_angular_integration_lebedev(39) = 47.433844543612018d0
 phi_angular_integration_lebedev(40) = 47.433844543612018d0
 phi_angular_integration_lebedev(41) = 132.566155456387975d0
 phi_angular_integration_lebedev(42) = 132.566155456387975d0
 phi_angular_integration_lebedev(43) = 47.433844543612018d0
 phi_angular_integration_lebedev(44) = 47.433844543612018d0
 phi_angular_integration_lebedev(45) = 132.566155456387975d0
 phi_angular_integration_lebedev(46) = 132.566155456387975d0
 phi_angular_integration_lebedev(47) = 47.433844543612018d0
 phi_angular_integration_lebedev(48) = 47.433844543612018d0
 phi_angular_integration_lebedev(49) = 132.566155456387975d0
 phi_angular_integration_lebedev(50) = 132.566155456387975d0
 phi_angular_integration_lebedev(51) = 36.187434472882487d0
 phi_angular_integration_lebedev(52) = 143.812565527117528d0
 phi_angular_integration_lebedev(53) = 36.187434472882487d0
 phi_angular_integration_lebedev(54) = 143.812565527117528d0
 phi_angular_integration_lebedev(55) = 36.187434472882487d0
 phi_angular_integration_lebedev(56) = 143.812565527117528d0
 phi_angular_integration_lebedev(57) = 36.187434472882487d0
 phi_angular_integration_lebedev(58) = 143.812565527117528d0
 phi_angular_integration_lebedev(59) = 65.323392119857786d0
 phi_angular_integration_lebedev(60) = 65.323392119857786d0
 phi_angular_integration_lebedev(61) = 114.676607880142214d0
 phi_angular_integration_lebedev(62) = 114.676607880142214d0
 phi_angular_integration_lebedev(63) = 65.323392119857786d0
 phi_angular_integration_lebedev(64) = 65.323392119857786d0
 phi_angular_integration_lebedev(65) = 114.676607880142214d0
 phi_angular_integration_lebedev(66) = 114.676607880142214d0
 phi_angular_integration_lebedev(67) = 65.323392119857786d0
 phi_angular_integration_lebedev(68) = 65.323392119857786d0
 phi_angular_integration_lebedev(69) = 114.676607880142214d0
 phi_angular_integration_lebedev(70) = 114.676607880142214d0
 phi_angular_integration_lebedev(71) = 65.323392119857786d0
 phi_angular_integration_lebedev(72) = 65.323392119857786d0
 phi_angular_integration_lebedev(73) = 114.676607880142214d0
 phi_angular_integration_lebedev(74) = 114.676607880142214d0
 phi_angular_integration_lebedev(75) = 12.867249998265978d0
 phi_angular_integration_lebedev(76) = 167.132750001734024d0
 phi_angular_integration_lebedev(77) = 12.867249998265978d0
 phi_angular_integration_lebedev(78) = 167.132750001734024d0
 phi_angular_integration_lebedev(79) = 12.867249998265978d0
 phi_angular_integration_lebedev(80) = 167.132750001734024d0
 phi_angular_integration_lebedev(81) = 12.867249998265978d0
 phi_angular_integration_lebedev(82) = 167.132750001734024d0
 phi_angular_integration_lebedev(83) = 80.940058987634842d0
 phi_angular_integration_lebedev(84) = 80.940058987634842d0
 phi_angular_integration_lebedev(85) = 99.059941012365158d0
 phi_angular_integration_lebedev(86) = 99.059941012365158d0
 phi_angular_integration_lebedev(87) = 80.940058987634842d0
 phi_angular_integration_lebedev(88) = 80.940058987634842d0
 phi_angular_integration_lebedev(89) = 99.059941012365158d0
 phi_angular_integration_lebedev(90) = 99.059941012365158d0
 phi_angular_integration_lebedev(91) = 80.940058987634842d0
 phi_angular_integration_lebedev(92) = 80.940058987634842d0
 phi_angular_integration_lebedev(93) = 99.059941012365158d0
 phi_angular_integration_lebedev(94) = 99.059941012365158d0
 phi_angular_integration_lebedev(95) = 80.940058987634842d0
 phi_angular_integration_lebedev(96) = 80.940058987634842d0
 phi_angular_integration_lebedev(97) = 99.059941012365158d0
 phi_angular_integration_lebedev(98) = 99.059941012365158d0
 phi_angular_integration_lebedev(99) = 28.082582278464621d0
 phi_angular_integration_lebedev(100) = 151.917417721535401d0
 phi_angular_integration_lebedev(101) = 28.082582278464621d0
 phi_angular_integration_lebedev(102) = 151.917417721535401d0
 phi_angular_integration_lebedev(103) = 28.082582278464621d0
 phi_angular_integration_lebedev(104) = 151.917417721535401d0
 phi_angular_integration_lebedev(105) = 28.082582278464621d0
 phi_angular_integration_lebedev(106) = 151.917417721535401d0
 phi_angular_integration_lebedev(107) = 63.299112780691708d0
 phi_angular_integration_lebedev(108) = 116.700887219308299d0
 phi_angular_integration_lebedev(109) = 63.299112780691708d0
 phi_angular_integration_lebedev(110) = 116.700887219308299d0
 phi_angular_integration_lebedev(111) = 63.299112780691708d0
 phi_angular_integration_lebedev(112) = 116.700887219308299d0
 phi_angular_integration_lebedev(113) = 63.299112780691708d0
 phi_angular_integration_lebedev(114) = 116.700887219308299d0
 phi_angular_integration_lebedev(115) = 28.082582278464621d0
 phi_angular_integration_lebedev(116) = 151.917417721535401d0
 phi_angular_integration_lebedev(117) = 28.082582278464621d0
 phi_angular_integration_lebedev(118) = 151.917417721535401d0
 phi_angular_integration_lebedev(119) = 28.082582278464621d0
 phi_angular_integration_lebedev(120) = 151.917417721535401d0
 phi_angular_integration_lebedev(121) = 28.082582278464621d0
 phi_angular_integration_lebedev(122) = 151.917417721535401d0
 phi_angular_integration_lebedev(123) = 81.931588861066530d0
 phi_angular_integration_lebedev(124) = 98.068411138933484d0
 phi_angular_integration_lebedev(125) = 81.931588861066530d0
 phi_angular_integration_lebedev(126) = 98.068411138933484d0
 phi_angular_integration_lebedev(127) = 81.931588861066530d0
 phi_angular_integration_lebedev(128) = 98.068411138933484d0
 phi_angular_integration_lebedev(129) = 81.931588861066530d0
 phi_angular_integration_lebedev(130) = 98.068411138933484d0
 phi_angular_integration_lebedev(131) = 63.299112780691708d0
 phi_angular_integration_lebedev(132) = 116.700887219308299d0
 phi_angular_integration_lebedev(133) = 63.299112780691708d0
 phi_angular_integration_lebedev(134) = 116.700887219308299d0
 phi_angular_integration_lebedev(135) = 63.299112780691708d0
 phi_angular_integration_lebedev(136) = 116.700887219308299d0
 phi_angular_integration_lebedev(137) = 63.299112780691708d0
 phi_angular_integration_lebedev(138) = 116.700887219308299d0
 phi_angular_integration_lebedev(139) = 81.931588861066530d0
 phi_angular_integration_lebedev(140) = 98.068411138933484d0
 phi_angular_integration_lebedev(141) = 81.931588861066530d0
 phi_angular_integration_lebedev(142) = 98.068411138933484d0
 phi_angular_integration_lebedev(143) = 81.931588861066530d0
 phi_angular_integration_lebedev(144) = 98.068411138933484d0
 phi_angular_integration_lebedev(145) = 81.931588861066530d0
 phi_angular_integration_lebedev(146) = 98.068411138933484d0

 weights_angular_integration_lebedev(1) = 0.000599631368862d0
 weights_angular_integration_lebedev(2) = 0.000599631368862d0
 weights_angular_integration_lebedev(3) = 0.000599631368862d0
 weights_angular_integration_lebedev(4) = 0.000599631368862d0
 weights_angular_integration_lebedev(5) = 0.000599631368862d0
 weights_angular_integration_lebedev(6) = 0.000599631368862d0
 weights_angular_integration_lebedev(7) = 0.007372999718621d0
 weights_angular_integration_lebedev(8) = 0.007372999718621d0
 weights_angular_integration_lebedev(9) = 0.007372999718621d0
 weights_angular_integration_lebedev(10) = 0.007372999718621d0
 weights_angular_integration_lebedev(11) = 0.007372999718621d0
 weights_angular_integration_lebedev(12) = 0.007372999718621d0
 weights_angular_integration_lebedev(13) = 0.007372999718621d0
 weights_angular_integration_lebedev(14) = 0.007372999718621d0
 weights_angular_integration_lebedev(15) = 0.007372999718621d0
 weights_angular_integration_lebedev(16) = 0.007372999718621d0
 weights_angular_integration_lebedev(17) = 0.007372999718621d0
 weights_angular_integration_lebedev(18) = 0.007372999718621d0
 weights_angular_integration_lebedev(19) = 0.007210515360144d0
 weights_angular_integration_lebedev(20) = 0.007210515360144d0
 weights_angular_integration_lebedev(21) = 0.007210515360144d0
 weights_angular_integration_lebedev(22) = 0.007210515360144d0
 weights_angular_integration_lebedev(23) = 0.007210515360144d0
 weights_angular_integration_lebedev(24) = 0.007210515360144d0
 weights_angular_integration_lebedev(25) = 0.007210515360144d0
 weights_angular_integration_lebedev(26) = 0.007210515360144d0
 weights_angular_integration_lebedev(27) = 0.007116355493118d0
 weights_angular_integration_lebedev(28) = 0.007116355493118d0
 weights_angular_integration_lebedev(29) = 0.007116355493118d0
 weights_angular_integration_lebedev(30) = 0.007116355493118d0
 weights_angular_integration_lebedev(31) = 0.007116355493118d0
 weights_angular_integration_lebedev(32) = 0.007116355493118d0
 weights_angular_integration_lebedev(33) = 0.007116355493118d0
 weights_angular_integration_lebedev(34) = 0.007116355493118d0
 weights_angular_integration_lebedev(35) = 0.007116355493118d0
 weights_angular_integration_lebedev(36) = 0.007116355493118d0
 weights_angular_integration_lebedev(37) = 0.007116355493118d0
 weights_angular_integration_lebedev(38) = 0.007116355493118d0
 weights_angular_integration_lebedev(39) = 0.007116355493118d0
 weights_angular_integration_lebedev(40) = 0.007116355493118d0
 weights_angular_integration_lebedev(41) = 0.007116355493118d0
 weights_angular_integration_lebedev(42) = 0.007116355493118d0
 weights_angular_integration_lebedev(43) = 0.007116355493118d0
 weights_angular_integration_lebedev(44) = 0.007116355493118d0
 weights_angular_integration_lebedev(45) = 0.007116355493118d0
 weights_angular_integration_lebedev(46) = 0.007116355493118d0
 weights_angular_integration_lebedev(47) = 0.007116355493118d0
 weights_angular_integration_lebedev(48) = 0.007116355493118d0
 weights_angular_integration_lebedev(49) = 0.007116355493118d0
 weights_angular_integration_lebedev(50) = 0.007116355493118d0
 weights_angular_integration_lebedev(51) = 0.006753829486314d0
 weights_angular_integration_lebedev(52) = 0.006753829486314d0
 weights_angular_integration_lebedev(53) = 0.006753829486314d0
 weights_angular_integration_lebedev(54) = 0.006753829486314d0
 weights_angular_integration_lebedev(55) = 0.006753829486314d0
 weights_angular_integration_lebedev(56) = 0.006753829486314d0
 weights_angular_integration_lebedev(57) = 0.006753829486314d0
 weights_angular_integration_lebedev(58) = 0.006753829486314d0
 weights_angular_integration_lebedev(59) = 0.006753829486314d0
 weights_angular_integration_lebedev(60) = 0.006753829486314d0
 weights_angular_integration_lebedev(61) = 0.006753829486314d0
 weights_angular_integration_lebedev(62) = 0.006753829486314d0
 weights_angular_integration_lebedev(63) = 0.006753829486314d0
 weights_angular_integration_lebedev(64) = 0.006753829486314d0
 weights_angular_integration_lebedev(65) = 0.006753829486314d0
 weights_angular_integration_lebedev(66) = 0.006753829486314d0
 weights_angular_integration_lebedev(67) = 0.006753829486314d0
 weights_angular_integration_lebedev(68) = 0.006753829486314d0
 weights_angular_integration_lebedev(69) = 0.006753829486314d0
 weights_angular_integration_lebedev(70) = 0.006753829486314d0
 weights_angular_integration_lebedev(71) = 0.006753829486314d0
 weights_angular_integration_lebedev(72) = 0.006753829486314d0
 weights_angular_integration_lebedev(73) = 0.006753829486314d0
 weights_angular_integration_lebedev(74) = 0.006753829486314d0
 weights_angular_integration_lebedev(75) = 0.007574394159054d0
 weights_angular_integration_lebedev(76) = 0.007574394159054d0
 weights_angular_integration_lebedev(77) = 0.007574394159054d0
 weights_angular_integration_lebedev(78) = 0.007574394159054d0
 weights_angular_integration_lebedev(79) = 0.007574394159054d0
 weights_angular_integration_lebedev(80) = 0.007574394159054d0
 weights_angular_integration_lebedev(81) = 0.007574394159054d0
 weights_angular_integration_lebedev(82) = 0.007574394159054d0
 weights_angular_integration_lebedev(83) = 0.007574394159054d0
 weights_angular_integration_lebedev(84) = 0.007574394159054d0
 weights_angular_integration_lebedev(85) = 0.007574394159054d0
 weights_angular_integration_lebedev(86) = 0.007574394159054d0
 weights_angular_integration_lebedev(87) = 0.007574394159054d0
 weights_angular_integration_lebedev(88) = 0.007574394159054d0
 weights_angular_integration_lebedev(89) = 0.007574394159054d0
 weights_angular_integration_lebedev(90) = 0.007574394159054d0
 weights_angular_integration_lebedev(91) = 0.007574394159054d0
 weights_angular_integration_lebedev(92) = 0.007574394159054d0
 weights_angular_integration_lebedev(93) = 0.007574394159054d0
 weights_angular_integration_lebedev(94) = 0.007574394159054d0
 weights_angular_integration_lebedev(95) = 0.007574394159054d0
 weights_angular_integration_lebedev(96) = 0.007574394159054d0
 weights_angular_integration_lebedev(97) = 0.007574394159054d0
 weights_angular_integration_lebedev(98) = 0.007574394159054d0
 weights_angular_integration_lebedev(99) = 0.006991087353303d0
 weights_angular_integration_lebedev(100) = 0.006991087353303d0
 weights_angular_integration_lebedev(101) = 0.006991087353303d0
 weights_angular_integration_lebedev(102) = 0.006991087353303d0
 weights_angular_integration_lebedev(103) = 0.006991087353303d0
 weights_angular_integration_lebedev(104) = 0.006991087353303d0
 weights_angular_integration_lebedev(105) = 0.006991087353303d0
 weights_angular_integration_lebedev(106) = 0.006991087353303d0
 weights_angular_integration_lebedev(107) = 0.006991087353303d0
 weights_angular_integration_lebedev(108) = 0.006991087353303d0
 weights_angular_integration_lebedev(109) = 0.006991087353303d0
 weights_angular_integration_lebedev(110) = 0.006991087353303d0
 weights_angular_integration_lebedev(111) = 0.006991087353303d0
 weights_angular_integration_lebedev(112) = 0.006991087353303d0
 weights_angular_integration_lebedev(113) = 0.006991087353303d0
 weights_angular_integration_lebedev(114) = 0.006991087353303d0
 weights_angular_integration_lebedev(115) = 0.006991087353303d0
 weights_angular_integration_lebedev(116) = 0.006991087353303d0
 weights_angular_integration_lebedev(117) = 0.006991087353303d0
 weights_angular_integration_lebedev(118) = 0.006991087353303d0
 weights_angular_integration_lebedev(119) = 0.006991087353303d0
 weights_angular_integration_lebedev(120) = 0.006991087353303d0
 weights_angular_integration_lebedev(121) = 0.006991087353303d0
 weights_angular_integration_lebedev(122) = 0.006991087353303d0
 weights_angular_integration_lebedev(123) = 0.006991087353303d0
 weights_angular_integration_lebedev(124) = 0.006991087353303d0
 weights_angular_integration_lebedev(125) = 0.006991087353303d0
 weights_angular_integration_lebedev(126) = 0.006991087353303d0
 weights_angular_integration_lebedev(127) = 0.006991087353303d0
 weights_angular_integration_lebedev(128) = 0.006991087353303d0
 weights_angular_integration_lebedev(129) = 0.006991087353303d0
 weights_angular_integration_lebedev(130) = 0.006991087353303d0
 weights_angular_integration_lebedev(131) = 0.006991087353303d0
 weights_angular_integration_lebedev(132) = 0.006991087353303d0
 weights_angular_integration_lebedev(133) = 0.006991087353303d0
 weights_angular_integration_lebedev(134) = 0.006991087353303d0
 weights_angular_integration_lebedev(135) = 0.006991087353303d0
 weights_angular_integration_lebedev(136) = 0.006991087353303d0
 weights_angular_integration_lebedev(137) = 0.006991087353303d0
 weights_angular_integration_lebedev(138) = 0.006991087353303d0
 weights_angular_integration_lebedev(139) = 0.006991087353303d0
 weights_angular_integration_lebedev(140) = 0.006991087353303d0
 weights_angular_integration_lebedev(141) = 0.006991087353303d0
 weights_angular_integration_lebedev(142) = 0.006991087353303d0
 weights_angular_integration_lebedev(143) = 0.006991087353303d0
 weights_angular_integration_lebedev(144) = 0.006991087353303d0
 weights_angular_integration_lebedev(145) = 0.006991087353303d0
 weights_angular_integration_lebedev(146) = 0.006991087353303d0

 else if (degree_max_integration_lebedev == 21)then


 theta_angular_integration_lebedev(1) = 0.000000000000000d0
 theta_angular_integration_lebedev(2) = 180.000000000000000d0
 theta_angular_integration_lebedev(3) = 90.000000000000000d0
 theta_angular_integration_lebedev(4) = -90.000000000000000d0
 theta_angular_integration_lebedev(5) = 90.000000000000000d0
 theta_angular_integration_lebedev(6) = 90.000000000000000d0
 theta_angular_integration_lebedev(7) = 90.000000000000000d0
 theta_angular_integration_lebedev(8) = 90.000000000000000d0
 theta_angular_integration_lebedev(9) = -90.000000000000000d0
 theta_angular_integration_lebedev(10) = -90.000000000000000d0
 theta_angular_integration_lebedev(11) = 0.000000000000000d0
 theta_angular_integration_lebedev(12) = 0.000000000000000d0
 theta_angular_integration_lebedev(13) = 180.000000000000000d0
 theta_angular_integration_lebedev(14) = 180.000000000000000d0
 theta_angular_integration_lebedev(15) = 45.000000000000000d0
 theta_angular_integration_lebedev(16) = -45.000000000000000d0
 theta_angular_integration_lebedev(17) = 135.000000000000000d0
 theta_angular_integration_lebedev(18) = -135.000000000000000d0
 theta_angular_integration_lebedev(19) = 45.000000000000000d0
 theta_angular_integration_lebedev(20) = 45.000000000000000d0
 theta_angular_integration_lebedev(21) = -45.000000000000000d0
 theta_angular_integration_lebedev(22) = -45.000000000000000d0
 theta_angular_integration_lebedev(23) = 135.000000000000000d0
 theta_angular_integration_lebedev(24) = 135.000000000000000d0
 theta_angular_integration_lebedev(25) = -135.000000000000000d0
 theta_angular_integration_lebedev(26) = -135.000000000000000d0
 theta_angular_integration_lebedev(27) = 45.000000000000000d0
 theta_angular_integration_lebedev(28) = 45.000000000000000d0
 theta_angular_integration_lebedev(29) = -45.000000000000000d0
 theta_angular_integration_lebedev(30) = -45.000000000000000d0
 theta_angular_integration_lebedev(31) = 135.000000000000000d0
 theta_angular_integration_lebedev(32) = 135.000000000000000d0
 theta_angular_integration_lebedev(33) = -135.000000000000000d0
 theta_angular_integration_lebedev(34) = -135.000000000000000d0
 theta_angular_integration_lebedev(35) = 74.700964736308180d0
 theta_angular_integration_lebedev(36) = -74.700964736308180d0
 theta_angular_integration_lebedev(37) = 74.700964736308180d0
 theta_angular_integration_lebedev(38) = -74.700964736308180d0
 theta_angular_integration_lebedev(39) = 105.299035263691835d0
 theta_angular_integration_lebedev(40) = -105.299035263691835d0
 theta_angular_integration_lebedev(41) = 105.299035263691835d0
 theta_angular_integration_lebedev(42) = -105.299035263691835d0
 theta_angular_integration_lebedev(43) = 15.299035263691827d0
 theta_angular_integration_lebedev(44) = 164.700964736308180d0
 theta_angular_integration_lebedev(45) = 15.299035263691827d0
 theta_angular_integration_lebedev(46) = 164.700964736308180d0
 theta_angular_integration_lebedev(47) = -15.299035263691827d0
 theta_angular_integration_lebedev(48) = -164.700964736308180d0
 theta_angular_integration_lebedev(49) = -15.299035263691827d0
 theta_angular_integration_lebedev(50) = -164.700964736308180d0
 theta_angular_integration_lebedev(51) = 45.000000000000000d0
 theta_angular_integration_lebedev(52) = 45.000000000000000d0
 theta_angular_integration_lebedev(53) = -45.000000000000000d0
 theta_angular_integration_lebedev(54) = -45.000000000000000d0
 theta_angular_integration_lebedev(55) = 135.000000000000000d0
 theta_angular_integration_lebedev(56) = 135.000000000000000d0
 theta_angular_integration_lebedev(57) = -135.000000000000000d0
 theta_angular_integration_lebedev(58) = -135.000000000000000d0
 theta_angular_integration_lebedev(59) = 24.038924410664215d0
 theta_angular_integration_lebedev(60) = -24.038924410664215d0
 theta_angular_integration_lebedev(61) = 24.038924410664215d0
 theta_angular_integration_lebedev(62) = -24.038924410664215d0
 theta_angular_integration_lebedev(63) = 155.961075589335792d0
 theta_angular_integration_lebedev(64) = -155.961075589335792d0
 theta_angular_integration_lebedev(65) = 155.961075589335792d0
 theta_angular_integration_lebedev(66) = -155.961075589335792d0
 theta_angular_integration_lebedev(67) = 65.961075589335792d0
 theta_angular_integration_lebedev(68) = 114.038924410664208d0
 theta_angular_integration_lebedev(69) = 65.961075589335792d0
 theta_angular_integration_lebedev(70) = 114.038924410664208d0
 theta_angular_integration_lebedev(71) = -65.961075589335792d0
 theta_angular_integration_lebedev(72) = -114.038924410664208d0
 theta_angular_integration_lebedev(73) = -65.961075589335792d0
 theta_angular_integration_lebedev(74) = -114.038924410664208d0
 theta_angular_integration_lebedev(75) = 45.000000000000000d0
 theta_angular_integration_lebedev(76) = 45.000000000000000d0
 theta_angular_integration_lebedev(77) = -45.000000000000000d0
 theta_angular_integration_lebedev(78) = -45.000000000000000d0
 theta_angular_integration_lebedev(79) = 135.000000000000000d0
 theta_angular_integration_lebedev(80) = 135.000000000000000d0
 theta_angular_integration_lebedev(81) = -135.000000000000000d0
 theta_angular_integration_lebedev(82) = -135.000000000000000d0
 theta_angular_integration_lebedev(83) = 61.389396297154533d0
 theta_angular_integration_lebedev(84) = -61.389396297154533d0
 theta_angular_integration_lebedev(85) = 61.389396297154533d0
 theta_angular_integration_lebedev(86) = -61.389396297154533d0
 theta_angular_integration_lebedev(87) = 118.610603702845481d0
 theta_angular_integration_lebedev(88) = -118.610603702845481d0
 theta_angular_integration_lebedev(89) = 118.610603702845481d0
 theta_angular_integration_lebedev(90) = -118.610603702845481d0
 theta_angular_integration_lebedev(91) = 28.610603702845474d0
 theta_angular_integration_lebedev(92) = 151.389396297154548d0
 theta_angular_integration_lebedev(93) = 28.610603702845474d0
 theta_angular_integration_lebedev(94) = 151.389396297154548d0
 theta_angular_integration_lebedev(95) = -28.610603702845474d0
 theta_angular_integration_lebedev(96) = -151.389396297154548d0
 theta_angular_integration_lebedev(97) = -28.610603702845474d0
 theta_angular_integration_lebedev(98) = -151.389396297154548d0
 theta_angular_integration_lebedev(99) = 74.847258028469312d0
 theta_angular_integration_lebedev(100) = -74.847258028469312d0
 theta_angular_integration_lebedev(101) = 105.152741971530688d0
 theta_angular_integration_lebedev(102) = -105.152741971530688d0
 theta_angular_integration_lebedev(103) = 15.152741971530682d0
 theta_angular_integration_lebedev(104) = -15.152741971530682d0
 theta_angular_integration_lebedev(105) = 164.847258028469327d0
 theta_angular_integration_lebedev(106) = -164.847258028469327d0
 theta_angular_integration_lebedev(107) = 0.000000000000000d0
 theta_angular_integration_lebedev(108) = 0.000000000000000d0
 theta_angular_integration_lebedev(109) = 180.000000000000000d0
 theta_angular_integration_lebedev(110) = 180.000000000000000d0
 theta_angular_integration_lebedev(111) = 0.000000000000000d0
 theta_angular_integration_lebedev(112) = 0.000000000000000d0
 theta_angular_integration_lebedev(113) = 180.000000000000000d0
 theta_angular_integration_lebedev(114) = 180.000000000000000d0
 theta_angular_integration_lebedev(115) = 90.000000000000000d0
 theta_angular_integration_lebedev(116) = 90.000000000000000d0
 theta_angular_integration_lebedev(117) = -90.000000000000000d0
 theta_angular_integration_lebedev(118) = -90.000000000000000d0
 theta_angular_integration_lebedev(119) = 90.000000000000000d0
 theta_angular_integration_lebedev(120) = 90.000000000000000d0
 theta_angular_integration_lebedev(121) = -90.000000000000000d0
 theta_angular_integration_lebedev(122) = -90.000000000000000d0
 theta_angular_integration_lebedev(123) = 16.165797434147173d0
 theta_angular_integration_lebedev(124) = 16.165797434147173d0
 theta_angular_integration_lebedev(125) = -16.165797434147173d0
 theta_angular_integration_lebedev(126) = -16.165797434147173d0
 theta_angular_integration_lebedev(127) = 163.834202565852820d0
 theta_angular_integration_lebedev(128) = 163.834202565852820d0
 theta_angular_integration_lebedev(129) = -163.834202565852820d0
 theta_angular_integration_lebedev(130) = -163.834202565852820d0
 theta_angular_integration_lebedev(131) = 59.711747172116482d0
 theta_angular_integration_lebedev(132) = 59.711747172116482d0
 theta_angular_integration_lebedev(133) = -59.711747172116482d0
 theta_angular_integration_lebedev(134) = -59.711747172116482d0
 theta_angular_integration_lebedev(135) = 120.288252827883525d0
 theta_angular_integration_lebedev(136) = 120.288252827883525d0
 theta_angular_integration_lebedev(137) = -120.288252827883525d0
 theta_angular_integration_lebedev(138) = -120.288252827883525d0
 theta_angular_integration_lebedev(139) = 73.834202565852820d0
 theta_angular_integration_lebedev(140) = 73.834202565852820d0
 theta_angular_integration_lebedev(141) = -73.834202565852820d0
 theta_angular_integration_lebedev(142) = -73.834202565852820d0
 theta_angular_integration_lebedev(143) = 106.165797434147180d0
 theta_angular_integration_lebedev(144) = 106.165797434147180d0
 theta_angular_integration_lebedev(145) = -106.165797434147180d0
 theta_angular_integration_lebedev(146) = -106.165797434147180d0
 theta_angular_integration_lebedev(147) = 80.390256591537280d0
 theta_angular_integration_lebedev(148) = 80.390256591537280d0
 theta_angular_integration_lebedev(149) = -80.390256591537280d0
 theta_angular_integration_lebedev(150) = -80.390256591537280d0
 theta_angular_integration_lebedev(151) = 99.609743408462720d0
 theta_angular_integration_lebedev(152) = 99.609743408462720d0
 theta_angular_integration_lebedev(153) = -99.609743408462720d0
 theta_angular_integration_lebedev(154) = -99.609743408462720d0
 theta_angular_integration_lebedev(155) = 30.288252827883525d0
 theta_angular_integration_lebedev(156) = 30.288252827883525d0
 theta_angular_integration_lebedev(157) = -30.288252827883525d0
 theta_angular_integration_lebedev(158) = -30.288252827883525d0
 theta_angular_integration_lebedev(159) = 149.711747172116503d0
 theta_angular_integration_lebedev(160) = 149.711747172116503d0
 theta_angular_integration_lebedev(161) = -149.711747172116503d0
 theta_angular_integration_lebedev(162) = -149.711747172116503d0
 theta_angular_integration_lebedev(163) = 9.609743408462711d0
 theta_angular_integration_lebedev(164) = 9.609743408462711d0
 theta_angular_integration_lebedev(165) = -9.609743408462711d0
 theta_angular_integration_lebedev(166) = -9.609743408462711d0
 theta_angular_integration_lebedev(167) = 170.390256591537309d0
 theta_angular_integration_lebedev(168) = 170.390256591537309d0
 theta_angular_integration_lebedev(169) = -170.390256591537309d0
 theta_angular_integration_lebedev(170) = -170.390256591537309d0

 phi_angular_integration_lebedev(1) = 90.000000000000000d0
 phi_angular_integration_lebedev(2) = 90.000000000000000d0
 phi_angular_integration_lebedev(3) = 90.000000000000000d0
 phi_angular_integration_lebedev(4) = 90.000000000000000d0
 phi_angular_integration_lebedev(5) = 0.000000000000000d0
 phi_angular_integration_lebedev(6) = 180.000000000000000d0
 phi_angular_integration_lebedev(7) = 45.000000000000000d0
 phi_angular_integration_lebedev(8) = 135.000000000000000d0
 phi_angular_integration_lebedev(9) = 45.000000000000000d0
 phi_angular_integration_lebedev(10) = 135.000000000000000d0
 phi_angular_integration_lebedev(11) = 45.000000000000000d0
 phi_angular_integration_lebedev(12) = 135.000000000000000d0
 phi_angular_integration_lebedev(13) = 45.000000000000000d0
 phi_angular_integration_lebedev(14) = 135.000000000000000d0
 phi_angular_integration_lebedev(15) = 90.000000000000000d0
 phi_angular_integration_lebedev(16) = 90.000000000000000d0
 phi_angular_integration_lebedev(17) = 90.000000000000000d0
 phi_angular_integration_lebedev(18) = 90.000000000000000d0
 phi_angular_integration_lebedev(19) = 54.735610317245346d0
 phi_angular_integration_lebedev(20) = 125.264389682754654d0
 phi_angular_integration_lebedev(21) = 54.735610317245346d0
 phi_angular_integration_lebedev(22) = 125.264389682754654d0
 phi_angular_integration_lebedev(23) = 54.735610317245346d0
 phi_angular_integration_lebedev(24) = 125.264389682754654d0
 phi_angular_integration_lebedev(25) = 54.735610317245346d0
 phi_angular_integration_lebedev(26) = 125.264389682754654d0
 phi_angular_integration_lebedev(27) = 21.149433535916117d0
 phi_angular_integration_lebedev(28) = 158.850566464083897d0
 phi_angular_integration_lebedev(29) = 21.149433535916117d0
 phi_angular_integration_lebedev(30) = 158.850566464083897d0
 phi_angular_integration_lebedev(31) = 21.149433535916117d0
 phi_angular_integration_lebedev(32) = 158.850566464083897d0
 phi_angular_integration_lebedev(33) = 21.149433535916117d0
 phi_angular_integration_lebedev(34) = 158.850566464083897d0
 phi_angular_integration_lebedev(35) = 75.218992356362847d0
 phi_angular_integration_lebedev(36) = 75.218992356362847d0
 phi_angular_integration_lebedev(37) = 104.781007643637167d0
 phi_angular_integration_lebedev(38) = 104.781007643637167d0
 phi_angular_integration_lebedev(39) = 75.218992356362847d0
 phi_angular_integration_lebedev(40) = 75.218992356362847d0
 phi_angular_integration_lebedev(41) = 104.781007643637167d0
 phi_angular_integration_lebedev(42) = 104.781007643637167d0
 phi_angular_integration_lebedev(43) = 75.218992356362847d0
 phi_angular_integration_lebedev(44) = 75.218992356362847d0
 phi_angular_integration_lebedev(45) = 104.781007643637167d0
 phi_angular_integration_lebedev(46) = 104.781007643637167d0
 phi_angular_integration_lebedev(47) = 75.218992356362847d0
 phi_angular_integration_lebedev(48) = 75.218992356362847d0
 phi_angular_integration_lebedev(49) = 104.781007643637167d0
 phi_angular_integration_lebedev(50) = 104.781007643637167d0
 phi_angular_integration_lebedev(51) = 72.494725487196703d0
 phi_angular_integration_lebedev(52) = 107.505274512803297d0
 phi_angular_integration_lebedev(53) = 72.494725487196703d0
 phi_angular_integration_lebedev(54) = 107.505274512803297d0
 phi_angular_integration_lebedev(55) = 72.494725487196703d0
 phi_angular_integration_lebedev(56) = 107.505274512803297d0
 phi_angular_integration_lebedev(57) = 72.494725487196703d0
 phi_angular_integration_lebedev(58) = 107.505274512803297d0
 phi_angular_integration_lebedev(59) = 47.595518437516212d0
 phi_angular_integration_lebedev(60) = 47.595518437516212d0
 phi_angular_integration_lebedev(61) = 132.404481562483795d0
 phi_angular_integration_lebedev(62) = 132.404481562483795d0
 phi_angular_integration_lebedev(63) = 47.595518437516212d0
 phi_angular_integration_lebedev(64) = 47.595518437516212d0
 phi_angular_integration_lebedev(65) = 132.404481562483795d0
 phi_angular_integration_lebedev(66) = 132.404481562483795d0
 phi_angular_integration_lebedev(67) = 47.595518437516212d0
 phi_angular_integration_lebedev(68) = 47.595518437516212d0
 phi_angular_integration_lebedev(69) = 132.404481562483795d0
 phi_angular_integration_lebedev(70) = 132.404481562483795d0
 phi_angular_integration_lebedev(71) = 47.595518437516212d0
 phi_angular_integration_lebedev(72) = 47.595518437516212d0
 phi_angular_integration_lebedev(73) = 132.404481562483795d0
 phi_angular_integration_lebedev(74) = 132.404481562483795d0
 phi_angular_integration_lebedev(75) = 37.646372670621822d0
 phi_angular_integration_lebedev(76) = 142.353627329378185d0
 phi_angular_integration_lebedev(77) = 37.646372670621822d0
 phi_angular_integration_lebedev(78) = 142.353627329378185d0
 phi_angular_integration_lebedev(79) = 37.646372670621822d0
 phi_angular_integration_lebedev(80) = 142.353627329378185d0
 phi_angular_integration_lebedev(81) = 37.646372670621822d0
 phi_angular_integration_lebedev(82) = 142.353627329378185d0
 phi_angular_integration_lebedev(83) = 64.412367834933889d0
 phi_angular_integration_lebedev(84) = 64.412367834933889d0
 phi_angular_integration_lebedev(85) = 115.587632165066111d0
 phi_angular_integration_lebedev(86) = 115.587632165066111d0
 phi_angular_integration_lebedev(87) = 64.412367834933889d0
 phi_angular_integration_lebedev(88) = 64.412367834933889d0
 phi_angular_integration_lebedev(89) = 115.587632165066111d0
 phi_angular_integration_lebedev(90) = 115.587632165066111d0
 phi_angular_integration_lebedev(91) = 64.412367834933889d0
 phi_angular_integration_lebedev(92) = 64.412367834933889d0
 phi_angular_integration_lebedev(93) = 115.587632165066111d0
 phi_angular_integration_lebedev(94) = 115.587632165066111d0
 phi_angular_integration_lebedev(95) = 64.412367834933889d0
 phi_angular_integration_lebedev(96) = 64.412367834933889d0
 phi_angular_integration_lebedev(97) = 115.587632165066111d0
 phi_angular_integration_lebedev(98) = 115.587632165066111d0
 phi_angular_integration_lebedev(99) = 90.000000000000000d0
 phi_angular_integration_lebedev(100) = 90.000000000000000d0
 phi_angular_integration_lebedev(101) = 90.000000000000000d0
 phi_angular_integration_lebedev(102) = 90.000000000000000d0
 phi_angular_integration_lebedev(103) = 90.000000000000000d0
 phi_angular_integration_lebedev(104) = 90.000000000000000d0
 phi_angular_integration_lebedev(105) = 90.000000000000000d0
 phi_angular_integration_lebedev(106) = 90.000000000000000d0
 phi_angular_integration_lebedev(107) = 15.152741971530682d0
 phi_angular_integration_lebedev(108) = 164.847258028469327d0
 phi_angular_integration_lebedev(109) = 15.152741971530682d0
 phi_angular_integration_lebedev(110) = 164.847258028469327d0
 phi_angular_integration_lebedev(111) = 74.847258028469312d0
 phi_angular_integration_lebedev(112) = 105.152741971530688d0
 phi_angular_integration_lebedev(113) = 74.847258028469312d0
 phi_angular_integration_lebedev(114) = 105.152741971530688d0
 phi_angular_integration_lebedev(115) = 15.152741971530682d0
 phi_angular_integration_lebedev(116) = 164.847258028469327d0
 phi_angular_integration_lebedev(117) = 15.152741971530682d0
 phi_angular_integration_lebedev(118) = 164.847258028469327d0
 phi_angular_integration_lebedev(119) = 74.847258028469312d0
 phi_angular_integration_lebedev(120) = 105.152741971530688d0
 phi_angular_integration_lebedev(121) = 74.847258028469312d0
 phi_angular_integration_lebedev(122) = 105.152741971530688d0
 phi_angular_integration_lebedev(123) = 31.304744789300912d0
 phi_angular_integration_lebedev(124) = 148.695255210699116d0
 phi_angular_integration_lebedev(125) = 31.304744789300912d0
 phi_angular_integration_lebedev(126) = 148.695255210699116d0
 phi_angular_integration_lebedev(127) = 31.304744789300912d0
 phi_angular_integration_lebedev(128) = 148.695255210699116d0
 phi_angular_integration_lebedev(129) = 31.304744789300912d0
 phi_angular_integration_lebedev(130) = 148.695255210699116d0
 phi_angular_integration_lebedev(131) = 81.682230932533884d0
 phi_angular_integration_lebedev(132) = 98.317769067466131d0
 phi_angular_integration_lebedev(133) = 81.682230932533884d0
 phi_angular_integration_lebedev(134) = 98.317769067466131d0
 phi_angular_integration_lebedev(135) = 81.682230932533884d0
 phi_angular_integration_lebedev(136) = 98.317769067466131d0
 phi_angular_integration_lebedev(137) = 81.682230932533884d0
 phi_angular_integration_lebedev(138) = 98.317769067466131d0
 phi_angular_integration_lebedev(139) = 31.304744789300912d0
 phi_angular_integration_lebedev(140) = 148.695255210699116d0
 phi_angular_integration_lebedev(141) = 31.304744789300912d0
 phi_angular_integration_lebedev(142) = 148.695255210699116d0
 phi_angular_integration_lebedev(143) = 31.304744789300912d0
 phi_angular_integration_lebedev(144) = 148.695255210699116d0
 phi_angular_integration_lebedev(145) = 31.304744789300912d0
 phi_angular_integration_lebedev(146) = 148.695255210699116d0
 phi_angular_integration_lebedev(147) = 60.063141298984171d0
 phi_angular_integration_lebedev(148) = 119.936858701015836d0
 phi_angular_integration_lebedev(149) = 60.063141298984171d0
 phi_angular_integration_lebedev(150) = 119.936858701015836d0
 phi_angular_integration_lebedev(151) = 60.063141298984171d0
 phi_angular_integration_lebedev(152) = 119.936858701015836d0
 phi_angular_integration_lebedev(153) = 60.063141298984171d0
 phi_angular_integration_lebedev(154) = 119.936858701015836d0
 phi_angular_integration_lebedev(155) = 81.682230932533884d0
 phi_angular_integration_lebedev(156) = 98.317769067466131d0
 phi_angular_integration_lebedev(157) = 81.682230932533884d0
 phi_angular_integration_lebedev(158) = 98.317769067466131d0
 phi_angular_integration_lebedev(159) = 81.682230932533884d0
 phi_angular_integration_lebedev(160) = 98.317769067466131d0
 phi_angular_integration_lebedev(161) = 81.682230932533884d0
 phi_angular_integration_lebedev(162) = 98.317769067466131d0
 phi_angular_integration_lebedev(163) = 60.063141298984171d0
 phi_angular_integration_lebedev(164) = 119.936858701015836d0
 phi_angular_integration_lebedev(165) = 60.063141298984171d0
 phi_angular_integration_lebedev(166) = 119.936858701015836d0
 phi_angular_integration_lebedev(167) = 60.063141298984171d0
 phi_angular_integration_lebedev(168) = 119.936858701015836d0
 phi_angular_integration_lebedev(169) = 60.063141298984171d0
 phi_angular_integration_lebedev(170) = 119.936858701015836d0

 weights_angular_integration_lebedev(1) = 0.005544842902037d0
 weights_angular_integration_lebedev(2) = 0.005544842902037d0
 weights_angular_integration_lebedev(3) = 0.005544842902037d0
 weights_angular_integration_lebedev(4) = 0.005544842902037d0
 weights_angular_integration_lebedev(5) = 0.005544842902037d0
 weights_angular_integration_lebedev(6) = 0.005544842902037d0
 weights_angular_integration_lebedev(7) = 0.006071332770671d0
 weights_angular_integration_lebedev(8) = 0.006071332770671d0
 weights_angular_integration_lebedev(9) = 0.006071332770671d0
 weights_angular_integration_lebedev(10) = 0.006071332770671d0
 weights_angular_integration_lebedev(11) = 0.006071332770671d0
 weights_angular_integration_lebedev(12) = 0.006071332770671d0
 weights_angular_integration_lebedev(13) = 0.006071332770671d0
 weights_angular_integration_lebedev(14) = 0.006071332770671d0
 weights_angular_integration_lebedev(15) = 0.006071332770671d0
 weights_angular_integration_lebedev(16) = 0.006071332770671d0
 weights_angular_integration_lebedev(17) = 0.006071332770671d0
 weights_angular_integration_lebedev(18) = 0.006071332770671d0
 weights_angular_integration_lebedev(19) = 0.006383674773515d0
 weights_angular_integration_lebedev(20) = 0.006383674773515d0
 weights_angular_integration_lebedev(21) = 0.006383674773515d0
 weights_angular_integration_lebedev(22) = 0.006383674773515d0
 weights_angular_integration_lebedev(23) = 0.006383674773515d0
 weights_angular_integration_lebedev(24) = 0.006383674773515d0
 weights_angular_integration_lebedev(25) = 0.006383674773515d0
 weights_angular_integration_lebedev(26) = 0.006383674773515d0
 weights_angular_integration_lebedev(27) = 0.005183387587748d0
 weights_angular_integration_lebedev(28) = 0.005183387587748d0
 weights_angular_integration_lebedev(29) = 0.005183387587748d0
 weights_angular_integration_lebedev(30) = 0.005183387587748d0
 weights_angular_integration_lebedev(31) = 0.005183387587748d0
 weights_angular_integration_lebedev(32) = 0.005183387587748d0
 weights_angular_integration_lebedev(33) = 0.005183387587748d0
 weights_angular_integration_lebedev(34) = 0.005183387587748d0
 weights_angular_integration_lebedev(35) = 0.005183387587748d0
 weights_angular_integration_lebedev(36) = 0.005183387587748d0
 weights_angular_integration_lebedev(37) = 0.005183387587748d0
 weights_angular_integration_lebedev(38) = 0.005183387587748d0
 weights_angular_integration_lebedev(39) = 0.005183387587748d0
 weights_angular_integration_lebedev(40) = 0.005183387587748d0
 weights_angular_integration_lebedev(41) = 0.005183387587748d0
 weights_angular_integration_lebedev(42) = 0.005183387587748d0
 weights_angular_integration_lebedev(43) = 0.005183387587748d0
 weights_angular_integration_lebedev(44) = 0.005183387587748d0
 weights_angular_integration_lebedev(45) = 0.005183387587748d0
 weights_angular_integration_lebedev(46) = 0.005183387587748d0
 weights_angular_integration_lebedev(47) = 0.005183387587748d0
 weights_angular_integration_lebedev(48) = 0.005183387587748d0
 weights_angular_integration_lebedev(49) = 0.005183387587748d0
 weights_angular_integration_lebedev(50) = 0.005183387587748d0
 weights_angular_integration_lebedev(51) = 0.006317929009814d0
 weights_angular_integration_lebedev(52) = 0.006317929009814d0
 weights_angular_integration_lebedev(53) = 0.006317929009814d0
 weights_angular_integration_lebedev(54) = 0.006317929009814d0
 weights_angular_integration_lebedev(55) = 0.006317929009814d0
 weights_angular_integration_lebedev(56) = 0.006317929009814d0
 weights_angular_integration_lebedev(57) = 0.006317929009814d0
 weights_angular_integration_lebedev(58) = 0.006317929009814d0
 weights_angular_integration_lebedev(59) = 0.006317929009814d0
 weights_angular_integration_lebedev(60) = 0.006317929009814d0
 weights_angular_integration_lebedev(61) = 0.006317929009814d0
 weights_angular_integration_lebedev(62) = 0.006317929009814d0
 weights_angular_integration_lebedev(63) = 0.006317929009814d0
 weights_angular_integration_lebedev(64) = 0.006317929009814d0
 weights_angular_integration_lebedev(65) = 0.006317929009814d0
 weights_angular_integration_lebedev(66) = 0.006317929009814d0
 weights_angular_integration_lebedev(67) = 0.006317929009814d0
 weights_angular_integration_lebedev(68) = 0.006317929009814d0
 weights_angular_integration_lebedev(69) = 0.006317929009814d0
 weights_angular_integration_lebedev(70) = 0.006317929009814d0
 weights_angular_integration_lebedev(71) = 0.006317929009814d0
 weights_angular_integration_lebedev(72) = 0.006317929009814d0
 weights_angular_integration_lebedev(73) = 0.006317929009814d0
 weights_angular_integration_lebedev(74) = 0.006317929009814d0
 weights_angular_integration_lebedev(75) = 0.006201670006589d0
 weights_angular_integration_lebedev(76) = 0.006201670006589d0
 weights_angular_integration_lebedev(77) = 0.006201670006589d0
 weights_angular_integration_lebedev(78) = 0.006201670006589d0
 weights_angular_integration_lebedev(79) = 0.006201670006589d0
 weights_angular_integration_lebedev(80) = 0.006201670006589d0
 weights_angular_integration_lebedev(81) = 0.006201670006589d0
 weights_angular_integration_lebedev(82) = 0.006201670006589d0
 weights_angular_integration_lebedev(83) = 0.006201670006589d0
 weights_angular_integration_lebedev(84) = 0.006201670006589d0
 weights_angular_integration_lebedev(85) = 0.006201670006589d0
 weights_angular_integration_lebedev(86) = 0.006201670006589d0
 weights_angular_integration_lebedev(87) = 0.006201670006589d0
 weights_angular_integration_lebedev(88) = 0.006201670006589d0
 weights_angular_integration_lebedev(89) = 0.006201670006589d0
 weights_angular_integration_lebedev(90) = 0.006201670006589d0
 weights_angular_integration_lebedev(91) = 0.006201670006589d0
 weights_angular_integration_lebedev(92) = 0.006201670006589d0
 weights_angular_integration_lebedev(93) = 0.006201670006589d0
 weights_angular_integration_lebedev(94) = 0.006201670006589d0
 weights_angular_integration_lebedev(95) = 0.006201670006589d0
 weights_angular_integration_lebedev(96) = 0.006201670006589d0
 weights_angular_integration_lebedev(97) = 0.006201670006589d0
 weights_angular_integration_lebedev(98) = 0.006201670006589d0
 weights_angular_integration_lebedev(99) = 0.005477143385137d0
 weights_angular_integration_lebedev(100) = 0.005477143385137d0
 weights_angular_integration_lebedev(101) = 0.005477143385137d0
 weights_angular_integration_lebedev(102) = 0.005477143385137d0
 weights_angular_integration_lebedev(103) = 0.005477143385137d0
 weights_angular_integration_lebedev(104) = 0.005477143385137d0
 weights_angular_integration_lebedev(105) = 0.005477143385137d0
 weights_angular_integration_lebedev(106) = 0.005477143385137d0
 weights_angular_integration_lebedev(107) = 0.005477143385137d0
 weights_angular_integration_lebedev(108) = 0.005477143385137d0
 weights_angular_integration_lebedev(109) = 0.005477143385137d0
 weights_angular_integration_lebedev(110) = 0.005477143385137d0
 weights_angular_integration_lebedev(111) = 0.005477143385137d0
 weights_angular_integration_lebedev(112) = 0.005477143385137d0
 weights_angular_integration_lebedev(113) = 0.005477143385137d0
 weights_angular_integration_lebedev(114) = 0.005477143385137d0
 weights_angular_integration_lebedev(115) = 0.005477143385137d0
 weights_angular_integration_lebedev(116) = 0.005477143385137d0
 weights_angular_integration_lebedev(117) = 0.005477143385137d0
 weights_angular_integration_lebedev(118) = 0.005477143385137d0
 weights_angular_integration_lebedev(119) = 0.005477143385137d0
 weights_angular_integration_lebedev(120) = 0.005477143385137d0
 weights_angular_integration_lebedev(121) = 0.005477143385137d0
 weights_angular_integration_lebedev(122) = 0.005477143385137d0
 weights_angular_integration_lebedev(123) = 0.005968383987681d0
 weights_angular_integration_lebedev(124) = 0.005968383987681d0
 weights_angular_integration_lebedev(125) = 0.005968383987681d0
 weights_angular_integration_lebedev(126) = 0.005968383987681d0
 weights_angular_integration_lebedev(127) = 0.005968383987681d0
 weights_angular_integration_lebedev(128) = 0.005968383987681d0
 weights_angular_integration_lebedev(129) = 0.005968383987681d0
 weights_angular_integration_lebedev(130) = 0.005968383987681d0
 weights_angular_integration_lebedev(131) = 0.005968383987681d0
 weights_angular_integration_lebedev(132) = 0.005968383987681d0
 weights_angular_integration_lebedev(133) = 0.005968383987681d0
 weights_angular_integration_lebedev(134) = 0.005968383987681d0
 weights_angular_integration_lebedev(135) = 0.005968383987681d0
 weights_angular_integration_lebedev(136) = 0.005968383987681d0
 weights_angular_integration_lebedev(137) = 0.005968383987681d0
 weights_angular_integration_lebedev(138) = 0.005968383987681d0
 weights_angular_integration_lebedev(139) = 0.005968383987681d0
 weights_angular_integration_lebedev(140) = 0.005968383987681d0
 weights_angular_integration_lebedev(141) = 0.005968383987681d0
 weights_angular_integration_lebedev(142) = 0.005968383987681d0
 weights_angular_integration_lebedev(143) = 0.005968383987681d0
 weights_angular_integration_lebedev(144) = 0.005968383987681d0
 weights_angular_integration_lebedev(145) = 0.005968383987681d0
 weights_angular_integration_lebedev(146) = 0.005968383987681d0
 weights_angular_integration_lebedev(147) = 0.005968383987681d0
 weights_angular_integration_lebedev(148) = 0.005968383987681d0
 weights_angular_integration_lebedev(149) = 0.005968383987681d0
 weights_angular_integration_lebedev(150) = 0.005968383987681d0
 weights_angular_integration_lebedev(151) = 0.005968383987681d0
 weights_angular_integration_lebedev(152) = 0.005968383987681d0
 weights_angular_integration_lebedev(153) = 0.005968383987681d0
 weights_angular_integration_lebedev(154) = 0.005968383987681d0
 weights_angular_integration_lebedev(155) = 0.005968383987681d0
 weights_angular_integration_lebedev(156) = 0.005968383987681d0
 weights_angular_integration_lebedev(157) = 0.005968383987681d0
 weights_angular_integration_lebedev(158) = 0.005968383987681d0
 weights_angular_integration_lebedev(159) = 0.005968383987681d0
 weights_angular_integration_lebedev(160) = 0.005968383987681d0
 weights_angular_integration_lebedev(161) = 0.005968383987681d0
 weights_angular_integration_lebedev(162) = 0.005968383987681d0
 weights_angular_integration_lebedev(163) = 0.005968383987681d0
 weights_angular_integration_lebedev(164) = 0.005968383987681d0
 weights_angular_integration_lebedev(165) = 0.005968383987681d0
 weights_angular_integration_lebedev(166) = 0.005968383987681d0
 weights_angular_integration_lebedev(167) = 0.005968383987681d0
 weights_angular_integration_lebedev(168) = 0.005968383987681d0
 weights_angular_integration_lebedev(169) = 0.005968383987681d0
 weights_angular_integration_lebedev(170) = 0.005968383987681d0
 endif
END_PROVIDER

